@@ -35,12 +35,13 @@ namespace Clasp {
 #if !defined(CLASP_USAGE)
 #define CLASP_USAGE   "clasp [number] [options] [file]"
 #endif
-#if !defined (SIGUSR1)
-#define SIGUSR1 SIGTERM
-#endif
-#if !defined(SIGUSR2)
-#define SIGUSR2 SIGTERM
+const int signals[] = { 
+	SIGINT, SIGTERM
+#if !defined(_WIN32)
+	, SIGUSR1, SIGUSR2, SIGQUIT, SIGHUP, SIGXCPU, SIGXFSZ
 #endif
+};
+
 inline std::ostream& operator << (std::ostream& os, Literal l) {
 	if (l.sign()) os << '-';
 	os << l.var();
@@ -128,13 +129,14 @@ void Application::kill(int sig) {
 		SCOPE_ALARM_LOCK();
 		INFO_OUT("clasp", "INTERRUPTED by signal!");
 		if (!facade_ || !facade_->terminate()) {
+			int status = E_INTERRUPT;
 			if (facade_ && facade_->state() != ClaspFacade::num_states) {
 				if (facade_->state() != ClaspFacade::state_start) { timer_[facade_->state()].stop(); }
 				timer_[ClaspFacade::state_start].stop();
 				cpuTotalTime_.stop();
-				printResult(sig);
+				status |= printResult(sig);
 			}
-			appTerminate(config_.ctx.enumerator()->enumerated > 0 ?  S_SATISFIABLE : S_UNKNOWN);
+			appTerminate(status);
 		}
 		else {                  // multiple threads are active - shutdown was initiated
 			INFO_OUT("clasp", "Shutting down threads...");
@@ -162,17 +164,10 @@ void Application::unblockSignals(bool deliverPending) {
 }
 
 void Application::installSigHandlers() {
-	if (signal(SIGINT, &Application::sigHandler) == SIG_IGN) {
-		signal(SIGINT, SIG_IGN);
-	}
-	if (signal(SIGTERM, &Application::sigHandler) == SIG_IGN) {
-		signal(SIGTERM, SIG_IGN);
-	}
-	if (SIGUSR1 != SIGTERM && (signal(SIGUSR1, &Application::sigHandler) == SIG_IGN)) {
-		signal(SIGUSR1, SIG_IGN);
-	}
-	if (SIGUSR2 != SIGTERM && (signal(SIGUSR2, &Application::sigHandler) == SIG_IGN)) {
-		signal(SIGUSR2, SIG_IGN);
+	for (int i = 0; i != sizeof(signals)/sizeof(int); ++i) {
+		if (signal(signals[i], &Application::sigHandler) == SIG_IGN) {
+			signal(signals[i], SIG_IGN);
+		}
 	}
 	if (app_.timeout > 0) {
 		setAlarmHandler(&Application::sigHandler);
@@ -289,22 +284,22 @@ int Application::run(int argc, char** argv) {
 		// command-line error
 		ERROR_OUT("clasp", messages.error.c_str());
 		INFO_OUT("clasp", "Try '--help' for usage information");
-		return S_ERROR;
+		return E_NO_RUN;
 	}
 	if (help || version) {
-		return EXIT_SUCCESS;
+		return E_UNKNOWN;
 	}
 	if (claspre_.listFeatures) {
 		claspre_.printFeatures();
-		return EXIT_SUCCESS;
+		return E_UNKNOWN;
 	}
 	if (clasp_.genTemplate) {
 		printTemplate();
-		return EXIT_SUCCESS;
+		return E_UNKNOWN;
 	}
 	if (!app_.lemmaIn.empty() && !isStdIn(app_.lemmaIn) && !std::ifstream(app_.lemmaIn.c_str()).is_open()) {
 		ERROR_OUT("clasp", "'lemma-in': could not open file!");
-		return EXIT_FAILURE;
+		return E_NO_RUN;
 	}
 	std::ofstream outF;
 	if (!app_.lemmaOut.empty()) {
@@ -312,12 +307,12 @@ int Application::run(int argc, char** argv) {
 		if (!isStdOut(app_.lemmaOut)) {
 			if (std::find(app_.input.begin(), app_.input.end(), app_.lemmaOut) != app_.input.end() || app_.lemmaIn == app_.lemmaOut) {
 				ERROR_OUT("clasp", "'lemma-out': cowardly refusing to overwrite input file!");
-				return EXIT_FAILURE;
+				return E_NO_RUN;
 			}
 			outF.open(app_.lemmaOut.c_str());
 			if (!outF.is_open()) {
 				ERROR_OUT("clasp", "'lemma-out': could not open file for writing!");
-				return EXIT_FAILURE;
+				return E_NO_RUN;
 			}
 			os = &outF;
 		}
@@ -325,7 +320,7 @@ int Application::run(int argc, char** argv) {
 		writeLemmas_->attach(config_.ctx);
 	}
 	installSigHandlers();
-	int retStatus = S_UNKNOWN;
+	int retStatus = E_UNKNOWN;
 	ClaspFacade clasp;
 	stats_.clear();
 	SolveStats agg;
@@ -349,13 +344,10 @@ int Application::run(int argc, char** argv) {
 		cpuTotalTime_.stop();
 		int sig = blockSignals();// disable signal handler
 		killAlarm();             // kill any pending alarms;
-		printResult(sig);
-		if      (clasp.result() == ClaspFacade::result_unsat) retStatus = S_UNSATISFIABLE;
-		else if (clasp.result() == ClaspFacade::result_sat)   retStatus = S_SATISFIABLE;
-		else                                                  retStatus = S_UNKNOWN;
+		retStatus = printResult(sig);
 	}
-	catch (const std::bad_alloc&  ) { retStatus = exception(S_MEMORY, "std::bad_alloc"); }
-	catch (const std::exception& e) { retStatus = exception(S_ERROR, e.what()); }
+	catch (const std::bad_alloc&  ) { retStatus = exception(E_MEMORY, "std::bad_alloc"); }
+	catch (const std::exception& e) { retStatus = exception(E_ERROR, e.what()); }
 	if    (app_.fastExit)           { appTerminate(retStatus);        }
 	else                            { fflush(stdout); fflush(stderr); }
 	return retStatus;
@@ -372,7 +364,7 @@ int Application::exception(int status, const char* what) {
 	ERROR_OUT("clasp", what);
 	if (facade_ && facade_->state() != ClaspFacade::num_states) {
 		cpuTotalTime_.stop();
-		printResult(status);
+		status |= printResult(status);
 	}
 	return status;
 }
@@ -382,7 +374,7 @@ int Application::exception(int status, const char* what) {
 // Generates a summary after search has stopped or has been interrupted.
 // The summary is then passed to the output object which is responsible
 // for printing.
-void Application::printResult(int sig) {
+int Application::printResult(int sig) {
 	OutputFormat::RunSummary sol(config_.ctx);
 	sol.termId       = sig;
 	sol.sig          = sig != 0;
@@ -395,7 +387,7 @@ void Application::printResult(int sig) {
 	sol.unsatTime    = sol.complete() && sol.solveTime-ttl >= 0.001 ? sol.solveTime-ttl : 0.0;
 	sol.cpuTime      = std::max(cpuTotalTime_.total(), 0.0);
 	if (config_.enumerate.onlyPre) {
-		if (sig) return;
+		if (sig) return E_INTERRUPT;
 		if (facade_->api()) { // asp-mode
 			facade_->result() == ClaspFacade::result_unsat
 				? (void)(std::cout << "0\n0\nB+\n1\n0\nB-\n1\n0\n0\n")
@@ -408,7 +400,7 @@ void Application::printResult(int sig) {
 			}
 			out_->reportResult(sol, 0, 0);
 		}
-		return;
+		return E_UNKNOWN;
 	}
 	if (!stats_.empty() && config_.numSolvers() > 1) {
 		// aggregate stats in stats_[0]
@@ -427,6 +419,7 @@ void Application::printResult(int sig) {
 		writeLemmas_->detach();
 		writeLemmas_.reset(0);
 	}
+	return sol.exitCode();
 }
 
 // State-transition callback called by ClaspFacade.
