@@ -41,13 +41,6 @@
 #define ERROR_OUT(sys,msg)   WRITE_STDERR("ERROR", (sys), (msg))
 #define INFO_OUT(sys,msg)    WRITE_STDERR("Info", (sys), (msg))
 #define WARNING_OUT(sys,msg) WRITE_STDERR("Warn", (sys), (msg))
-
-// exit codes
-#define S_SATISFIABLE   10  // problem is satisfiable
-#define S_UNSATISFIABLE 20  // problem was proved to be unsatisfiable
-#define S_UNKNOWN       0   // satisfiablity of problem not knwon; search was interrupted or did not start
-#define S_ERROR EXIT_FAILURE// internal error, except out of memory
-#define S_MEMORY        127 // out of memory
 /////////////////////////////////////////////////////////////////////////////////////////
 // Clasp::Application
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -135,7 +128,7 @@ private:
 	// Status information & output
 	void configureOutput(Input::Format f);
 	void model(const Solver& s, const Enumerator& e, bool cons);
-	void printResult(int sig);
+	int  printResult(int sig);
 	// -------------------------------------------------------------------------------------------  
 	typedef PodVector<const SolveStats*>::type StatsVec;
 	ClaspConfig                   config_;
