@@ -488,7 +488,7 @@ bool SearchOptions::mapReduceOpts(SearchOptions* this_, const std::string& n, co
 		}
 	}
 	else if (n == "del-algo") {
-		std::pair<std::string, uint32> algo("", p.strategy.score);
+		std::pair<std::string, uint32> algo("", (uint32)p.strategy.score);
 		int a;
 		ok = parse(value, algo) && mapEnumImpl(delAlgos, algo.first, a) 
 		  && SET_M(p.strategy.score, algo.second, 2)
