@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2010, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -32,6 +32,12 @@
 #include <clasp/solver.h>
 #include <iosfwd>
 
+namespace ProgramOptions {
+struct StringSlice;
+// Specialized function for mapping unsigned integers
+// Parses numbers >= 0, -1, and the string umax
+StringSlice parseValue(const StringSlice& in, uint32& x, int extra);
+}
 namespace Clasp {
 
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -45,40 +51,20 @@ struct StringToEnum {
 };
 typedef StringToEnum EnumMap;
 
-// Group "Basic Options"
-struct BasicOptions {
-	BasicOptions();
-	typedef std::pair<int, int> QPair;
-	typedef ProgramOptions::StringSeq StringSeq;
-	static bool mapIfs(const std::string& s, char& ifs);
-	void initOptions(ProgramOptions::OptionContext& root);
-	StringSeq   input;  // list of input files - only first used!
-	int         verbose;// verbosity level
-	int         timeout;// timeout in seconds (default: none=-1)
-	QPair       quiet;  // configure printing of models and optimization values
-	int         stats;  // print statistics
-	int         outf;   // output format
-	bool        maxSat; // interpret dimacs as MaxSAT problem
-	char        ifs;    // field separator
-	enum OutputFormat {
-		out_def     = 0,
-		out_comp    = 1,
-		out_json    = 2
-	};
-};
-
 // Group "Clasp - General Options"
 // Options of this group are mapped to ClaspConfig::api
 // and ClaspConfig::enumerate
 struct GeneralOptions {
-	explicit GeneralOptions(ClaspConfig* c = 0) : config(c) {}
-	static bool parseEnumMode(const std::string& s, GlobalOptions::EnumMode& m); 
-	static bool mapMyVec(GeneralOptions*, const std::string&, const std::vector<int>* v);
-	static bool mapLearnExplicit(GeneralOptions*, const std::string& k, const bool* b);
+	explicit GeneralOptions(ClaspConfig* c = 0) : config(c), defConfig(0) {}
+	static bool mapOptVal(GeneralOptions*, const std::string&, const std::string&);
+	static bool mapSatPre(GeneralOptions*, const std::string&, const std::string&);
+	static bool mapLearnExplicit(GeneralOptions*, const std::string&, const std::string&);
+	static bool mapDefaultConfig(GeneralOptions*, const std::string&, const std::string&);
+	static bool parseSolveLimit(const std::string& s, SolveLimits& limit);
 	void initOptions(ProgramOptions::OptionContext& root);
-	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
-	std::vector<int>  myVec;
+	bool validateOptions(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	ClaspConfig*      config;
+	const char*       defConfig;
 	static const EnumMap enumModes[];
 	static const EnumMap extRules[];
 };
@@ -87,52 +73,60 @@ struct GeneralOptions {
 // Options of these groups are mapped to ClaspConfig::solve 
 // and ClaspConfig::solver
 struct SearchOptions {
-	explicit SearchOptions(LocalOptions* l);
+	explicit SearchOptions(SolverConfig* l);
 	void initOptions(ProgramOptions::OptionContext& root);
-	bool validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	void initOptions(const ProgramOptions::OptionContext& root, ProgramOptions::OptionContext& local, SolverConfig* x, uint32 delOpts = 1);
+	bool validateOptions(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	// value parsing 
-	static bool parseHeuristic(const std::string& s, std::string&);
-	static bool parseSchedule(const std::string& s, ScheduleStrategy& sched);
+	static bool parseSchedule(const std::string& s, ScheduleStrategy& sched, bool allowNo);
 	// value mapping
 	static bool mapSolverOpts(SearchOptions*, const std::string& opt, const std::string& v);
-	static bool mapPair(SearchOptions*      , const std::string& opt, const std::string& v);
+	static bool mapHeuOpts(SearchOptions*, const std::string& opt, const std::string& v);
 	static bool mapRestart(SearchOptions*   , const std::string& opt, const std::string& v);
-	static bool mapNumber(SearchOptions*    , const std::string& opt, const double* v);
 	static bool mapRandOpts(SearchOptions*  , const std::string& opt, const std::string& v);
 	static bool mapReduceOpts(SearchOptions*, const std::string& opt, const std::string& v);
-	LocalOptions*      local;
+	
+	SolverConfig*      local;
 	SolverStrategies*  solverOpts() const;
-	double             aNumber;
+	uint32             enabledDel;
+	static const EnumMap heuTypes[];
 	static const EnumMap lookTypes[];
-	static const EnumMap anteTypes[];
 	static const EnumMap loopTypes[];
+	static const EnumMap delAlgos[];
 };
 	
-#define CLASP_DEFAULT_PORTFOLIO_SIZE 9
+#define CLASP_DEFAULT_PORTFOLIO_SIZE 13
 extern const char* portfolio_g;
-
+extern const char* defConfigs_g;
+struct OptionConfig {
+	explicit OptionConfig(const char* raw) { initFromRaw(raw); }
+	void initFromRaw(const char* r);
+	std::string name;
+	std::string cmdLine;
+};
 // Combines all groups and drives initialization/validation 
 // of command-line options.
 class ClaspOptions {
 public:
 	ClaspOptions() : genTemplate(false), satPreDefault(true), config(0) {}
-	BasicOptions basic;
 	std::string  portfolio;
 	bool         genTemplate;
 	bool         satPreDefault;
 	void         initOptions(ProgramOptions::OptionContext& root, ClaspConfig& c);
-	bool         validateOptions(const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	bool         validateOptions(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	void         applyDefaults(Input::Format f);
 	const char*  getInputDefaults(Input::Format f) const;
-	static bool  mapThreadOpts(ClaspOptions*, const std::string& k, const std::string& v);
+	static bool  mapSolveOpts(ClaspOptions*, const std::string& k, const std::string& v);
 private:
-	bool parsePortfolio(std::istream& in, uint32 max, std::string& mem) const;
-	bool populateThreadConfigs(const char* port, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
-	bool parseConfig(const char* p, uint32 id, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	const char*  getPortfolio(std::string& mem) const;
+	bool applyDefaultConfig(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
+	bool applySearchConfig(const ProgramOptions::OptionContext& ctx, const OptionConfig& c, const ProgramOptions::ParsedOptions&, ProgramOptions::Messages&);
+	bool populateThreadConfigs(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions& parsed, ProgramOptions::Messages&);
 	void initThreadOptions(ProgramOptions::OptionContext& root);
-	ClaspConfig*                  config;
-	std::auto_ptr<GeneralOptions> mode;
-	std::auto_ptr<SearchOptions>  search;
+	void removeConfig(OptionConfig& cfg, const char* opt) const;
+	ClaspConfig*                   config;
+	std::auto_ptr<GeneralOptions>  mode;
+	std::auto_ptr<SearchOptions>   search;
 };
 }
 #endif
