@@ -61,6 +61,16 @@ OutputFormat::RunSummary::Result OutputFormat::RunSummary::result() const {
 	return r;
 }
 
+int OutputFormat::RunSummary::exitCode() const {
+	const Enumerator* e = ctx.enumerator();
+	uint64 m            = e->enumerated;
+	int   ec            = 0;
+	if (termSig()) { ec|= E_INTERRUPT; }
+	if (m != 0)    { ec|= E_SAT; }
+	if (complete()){ ec|= E_EXHAUST; }
+	return ec;
+}
+
 uint64 OutputFormat::RunSummary::models() const {
 	return (ctx.enumerator()->enumerated == 0 || !(ctx.enumerator()->optimize()||consequences)) ? ctx.enumerator()->enumerated : 1;
 }
@@ -148,7 +158,7 @@ OutputFormat::Model OutputFormat::storeModel(const Assignment& a) {
 // DefaultOutput
 /////////////////////////////////////////////////////////////////////////////////////////
 #define printKey(k)                  printf("%s%-*s: ", format_[cat_comment], w_, (k))
-#define printKeyValue(k, fmt, value) printf("%s%-*s: "fmt, format_[cat_comment], w_, (k), (value))
+#define printKeyValue(k, fmt, value) printf("%s%-*s: " fmt, format_[cat_comment], w_, (k), (value))
 const char* const lnSep = "----------------------------------------------------------------------------";
 
 DefaultOutput::DefaultOutput(uint32 v, const std::pair<uint32,uint32>& q, Format f, char ifs)
@@ -177,14 +187,19 @@ void DefaultOutput::configureFormat(Format f, PrintPair q, char ifs) {
 	format_[cat_atom_pre]  = "";
 	format_[cat_atom_post] = "";
 	if (f == format_aspcomp) {
+		format_[cat_comment]   = "% ";
+		format_[cat_value]     = "ANSWER\n";
+		format_[cat_objective] = "COST ";
 		format_[cat_atom_post] = ".";
-		format_[cat_objective] = "OPT: ";
-		result[RunSummary::result_sat]     = "ANSWER SET FOUND";
-		result[RunSummary::result_unsat]   = "INCONSISTENT";
+		result[RunSummary::result_sat]     = "";
 		result[RunSummary::result_sat_opt] = "";
-		setVerbosity(-1);
+		result[RunSummary::result_unsat]   = "INCONSISTENT";
+		result[RunSummary::result_optimum] = "OPTIMUM";
+		if (q.first == uint32(print_f_default)) {
+			q.first = print_best;
+		}
 		if (q.second == uint32(print_f_default)) {
-			q.second = print_no;
+			q.second= q.first;
 		}
 	}
 	else if (f == format_sat09 || f == format_pb09) {
@@ -275,7 +290,7 @@ void DefaultOutput::reportProgress(const SolveEvent& e) {
 				format_[cat_comment], lnSep, format_[cat_comment], format_[cat_comment], format_[cat_comment], lnSep);
 		}
 		else if (++ev_ == 2) { printf("%s%s\n", format_[cat_comment], lnSep); }
-		printf("%s[%u:%c]|%7u/%-7u|%8u/%-8u|%10"PRIu64"/%-6.3f|%8"PRId64"/%-8"PRId64"|\n"
+		printf("%s[%u:%c]|%7u/%-7u|%8u/%-8u|%10" PRIu64"/%-6.3f|%8" PRId64"/%-8" PRId64"|\n"
 			, format_[cat_comment]
 			, s.id()
 			, static_cast<char>(ev.evType)
@@ -381,8 +396,11 @@ void DefaultOutput::printOptimizeValues(const SharedMinimizeData& m) const {
 
 void DefaultOutput::printResult(const RunSummary& sol, const SolveStats** stats, size_t num) {
 	const Enumerator& en = *sol.ctx.enumerator();
-	printf("%s%s\n", format_[cat_result], result[sol.result()]);
-	if (verbosity() > (0 - (num!=0))) {
+	RunSummary::Result r = sol.result();
+	if (strcmp(result[r], "") != 0) {
+		printf("%s%s\n", format_[cat_result], result[r]);
+	}
+	if (verbosity() > 0 || num != 0) {
 		printf("\n");
 		if      (sol.termSig() == SIGALRM) { printKeyValue("TIME LIMIT", "%d\n", 1);  }
 		else if (sol.termSig() != 0)       { printKeyValue("INTERRUPTED", "%d\n", 1); }
@@ -395,7 +413,7 @@ void DefaultOutput::printResult(const RunSummary& sol, const SolveStats** stats,
 			printf("%-6s\n", buf);
 		}
 		else {
-			printf("%-6"PRIu64"\n", sol.models());
+			printf("%-6" PRIu64"\n", sol.models());
 		}
 		if (en.enumerated) {
 			if (en.enumerated != sol.models()) { 
@@ -421,7 +439,7 @@ void DefaultOutput::printResult(const RunSummary& sol, const SolveStats** stats,
 				, sol.unsatTime);
 		printKeyValue("CPU Time", "%.3fs\n", sol.cpuTime);
 	}
-	if (verbosity() >= 0 && num != 0) {
+	if (num != 0) {
 		printStats(sol, stats, num);
 	}
 }
@@ -445,7 +463,7 @@ void DefaultOutput::printStats(const RunSummary& sol, const SolveStats** stats,
 void DefaultOutput::printSolveStats(const SolveStats& st) const {
 	printKeyValue("Choices", "%" PRIu64"\n", st.choices);
 	printKeyValue("Conflicts", "%" PRIu64"\n", st.conflicts);
-	printKeyValue("Restarts", "%-6"PRIu64"", st.restarts);
+	printKeyValue("Restarts", "%-6" PRIu64"", st.restarts);
 	if (st.restarts) {
 		printf(" (Average: %.2f Last: %" PRIu64")", average(st.analyzed, st.restarts), st.cflLast);
 	}
@@ -496,18 +514,18 @@ void DefaultOutput::printProblemStats(const ProblemStats& ps) const {
 
 void DefaultOutput::printLemmaStats(const SolveStats& st) const {
 	uint64 sum   = std::accumulate(st.learnts, st.learnts+Constraint_t::max_value, uint64(0));
-	printKeyValue("Lemmas", "%-6"PRIu64, sum);
+	printKeyValue("Lemmas", "%-6" PRIu64, sum);
 	printf(" (Binary:%5.1f%% Ternary:%5.1f%% Other:%5.1f%%)\n"
 			, percent(st.binary, sum)
 			, percent(st.ternary, sum)
 			, percent(sum - st.binary - st.ternary, sum));
-	printKeyValue("  Conflict", "%-6"PRIu64, st.learnts[0]);
+	printKeyValue("  Conflict", "%-6" PRIu64, st.learnts[0]);
 	printf(" (Average Length: %.1f) \n", average(st.lits[0], st.learnts[0]));
-	printKeyValue("  Loop", "%-6"PRIu64, st.learnts[1]);
+	printKeyValue("  Loop", "%-6" PRIu64, st.learnts[1]);
 	printf(" (Average Length: %.1f) \n", average(st.lits[1], st.learnts[1]));
-	printKeyValue("  Other", "%-6"PRIu64, st.learnts[2]);
+	printKeyValue("  Other", "%-6" PRIu64, st.learnts[2]);
 	printf(" (Average Length: %.1f)\n"  , average(st.lits[2], st.learnts[2]));
-	printKeyValue("  Deleted", "%-6"PRIu64"\n", st.deleted);
+	printKeyValue("  Deleted", "%-6" PRIu64"\n", st.deleted);
 }
 
 void DefaultOutput::printJumpStats(const SolveStats& st) const {
@@ -515,10 +533,10 @@ void DefaultOutput::printJumpStats(const SolveStats& st) const {
 		const JumpStats& js = *st.jumps;
 		w_ += 8;
 		printf("\n");
-		printKeyValue("Backtracks", "%-6"PRIu64"\n", st.conflicts-st.analyzed);
-		printKeyValue("Backjumps", "%-6"PRIu64, st.analyzed);
+		printKeyValue("Backtracks", "%-6" PRIu64"\n", st.conflicts-st.analyzed);
+		printKeyValue("Backjumps", "%-6" PRIu64, st.analyzed);
 		printf(" (Bounded: %" PRIu64")\n", js.bJumps);
-		printKeyValue("Skippable Levels", "%-6"PRIu64, js.jumpSum);
+		printKeyValue("Skippable Levels", "%-6" PRIu64, js.jumpSum);
 		printf(" (Skipped: %" PRIu64" Rate: %5.1f%%)\n", js.jumpSum - js.boundSum, percent(js.jumpSum - js.boundSum, js.jumpSum));
 		printKeyValue("Max Jump Length", "%-6u", js.maxJump);
 		printf(" (Executed: %u)\n", js.maxJumpEx);
@@ -557,14 +575,14 @@ void DefaultOutput::printParallelStats(const SolveStats& stats, bool accu) const
 			printKeyValue("CPU Time",  "%.3fs\n", ps.cpuTime);
 			printKeyValue("Models", "%" PRIu64"\n", stats.models);
 		}
-		printKeyValue("Distributed", "%-6"PRIu64, ps.distributed);
+		printKeyValue("Distributed", "%-6" PRIu64, ps.distributed);
 		printf(" (Ratio: %.2f Average LBD: %.2f) \n", ps.distributed/std::max(1.0, double(sumLem)), average(ps.sumLbd, ps.distributed));
-		printKeyValue("Integrated", "%-6"PRIu64, ps.integrated);
+		printKeyValue("Integrated", "%-6" PRIu64, ps.integrated);
 		if (accu) { printf(" (Ratio: %.2f ", ps.integrated/std::max(1.0, double(ps.distributed))); }
 		else      { printf(" ("); }
 		printf("Unit: %" PRIu64" Average Jumps: %.2f)\n", ps.imps, average(ps.jumps, ps.imps));
 		printKeyValue("Splits", "%" PRIu64"\n", (uint64)ps.splits);
-		printKeyValue("Problems", "%-6"PRIu64, (uint64)ps.gps);
+		printKeyValue("Problems", "%-6" PRIu64, (uint64)ps.gps);
 		printf(" (Average Length: %.2f) \n\n", average(ps.gpLits, ps.gps));
 	}
 }
