@@ -247,6 +247,8 @@ echo "TARGET       := lib/${LIB_OPTS}.a"   >> $LIB_OPTS/.CONFIG
 echo "FLAGS        := ../FLAGS"            >> $LIB_CLASP/.CONFIG
 echo "FLAGS        := ../FLAGS"            >> $LIB_OPTS/.CONFIG
 echo "SOURCE_DIR   := \$(PROJECT_ROOT)/${LIB_CLASP}/src" >> $LIB_CLASP/.CONFIG
+echo "TEST_DIR     := \$(PROJECT_ROOT)/${LIB_CLASP}/tests" >> $LIB_CLASP/.CONFIG
+echo "TEST_TARGET  := ./test-lib"          >> $LIB_CLASP/.CONFIG
 echo "SOURCE_DIR   := \$(PROJECT_ROOT)/${LIB_OPTS}/src" >> $LIB_OPTS/.CONFIG
 echo "INCLUDE_DIR  := \$(PROJECT_ROOT)/${LIB_CLASP}/clasp" >> $LIB_CLASP/.CONFIG
 echo "INCLUDE_DIR  := \$(PROJECT_ROOT)/${LIB_OPTS}/program_opts" >> $LIB_OPTS/.CONFIG
