@@ -67,9 +67,9 @@ private:
 	SharedLiterals(const Literal* lits, uint32 size, ConstraintType t, uint32 numRefs);
 	SharedLiterals(const SharedLiterals&);
 	SharedLiterals& operator=(const SharedLiterals&);
-	std::atomic<int32> refCount_;
-	uint32             size_type_;
-	Literal            lits_[0];
+	Clasp::atomic<int32> refCount_;
+	uint32               size_type_;
+	Literal              lits_[0];
 };
 
 //! A helper-class for creating/adding clauses.
