@@ -38,7 +38,7 @@ namespace Clasp {
  */
 //@{
 
-//! A variable is currently nothing more but an integer in the range [0..varMax)
+//! A variable is currently nothing more but an integer in the range [0..varMax).
 typedef uint32 Var;
 
 //! varMax is not a valid variale, i.e. currently Clasp supports at most 2^30 variables.
@@ -47,7 +47,7 @@ const Var varMax = (Var(1) << 30);
 //! The variable 0 has a special meaning in the solver.
 const Var sentVar = 0;
 
-//! ids are integers in the range [0..idMax)
+//! Ids are integers in the range [0..idMax).
 const uint32 idMax = UINT32_MAX;
 
 //! Possible types of a variable.
@@ -62,9 +62,9 @@ struct Var_t {
 };
 typedef Var_t::Type VarType;
 
-//! A signed integer type used to represent weights
+//! A signed integer type used to represent weights.
 typedef int32 weight_t;
-//! A signed integer type used to represent sums of weights
+//! A signed integer type used to represent sums of weights.
 typedef int64 wsum_t;
 
 //! A literal is a variable or its negation.
@@ -82,9 +82,9 @@ public:
 	//! The default constructor creates the positive literal of the special sentinel var.
 	Literal() : rep_(0) { }
 	
-	//! creates a literal of the variable var with sign s
+	//! Creates a literal of the variable var with sign s.
 	/*!
-	 * \param var The literal's variable
+	 * \param var The literal's variable.
 	 * \param s true if new literal should be negative.
 	 * \pre var < varMax
 	 */
@@ -92,13 +92,13 @@ public:
 		assert( var < varMax );
 	}
 
-	//! returns the unique index of this literal
+	//! Returns the unique index of this literal.
 	/*!
 	 * \note The watch-flag is ignored and thus the index of a literal can be stored in 31-bits. 
 	 */
 	uint32 index() const { return rep_ >> 1; }
 	
-	//! creates a literal from an index.
+	//! Creates a literal from an index.
 	/*!
 	 * \pre idx < 2^31
 	 */
@@ -107,16 +107,16 @@ public:
 		return Literal(idx<<1);
 	}
 
-	//! creates a literal from an unsigned integer.
+	//! Creates a literal from an unsigned integer.
 	static Literal fromRep(uint32 rep) { return Literal(rep); }
 	
 	uint32& asUint()        { return rep_; }
 	uint32  asUint() const  { return rep_; }
 
-	//! returns the variable of the literal.
+	//! Returns the variable of the literal.
 	Var var() const { return rep_ >> 2; }
 	
-	//! returns the sign of the literal.
+	//! Returns the sign of the literal.
 	/*!
 	 * \return true if the literal is negative. Otherwise false.
 	 */
@@ -124,16 +124,16 @@ public:
 
 	void swap(Literal& other) { std::swap(rep_, other.rep_); }
 	
-	//! sets the watched-flag of this literal
+	//! Sets the watched-flag of this literal.
 	void watch() { store_set_bit(rep_, 0); }
 	
-	//! clears the watched-flag of this literal
+	//! Clears the watched-flag of this literal.
 	void clearWatch() { store_clear_bit(rep_, 0); }
 	
-	//! returns true if the watched-flag of this literal is set
+	//! Returns true if the watched-flag of this literal is set.
 	bool watched() const { return test_bit(rep_, 0); }
 
-	//! returns the complimentary literal of this literal.
+	//! Returns the complimentary literal of this literal.
 	/*!
 	 *  The complementary Literal of a Literal is a Literal referring to the
 	 *  same variable but with inverted sign.
@@ -161,25 +161,25 @@ private:
 // Common interface
 /////////////////////////////////////////////////////////////////////////////////////////
 
-//! creates the negative literal of variable v.
+//! Creates the negative literal of variable v.
 inline Literal negLit(Var v) { return Literal(v, true);}
-//! creates the positive literal of variable v.
+//! Creates the positive literal of variable v.
 inline Literal posLit(Var v) { return Literal(v, false);}
-//! returns the index of variable v.
+//! Returns the index of variable v.
 /*!
  * \note same as posLit(v).index()
  */
 inline uint32 index(Var v) { return v << 1; }
 
-//! returns true if p represents the special variable 0
+//! Returns true if p represents the special variable 0
 inline bool isSentinel(Literal p) { return p.var() == sentVar; }
 
-//! defines a strict-weak-ordering for Literals.
+//! Defines a strict-weak-ordering for Literals.
 inline bool operator<(const Literal& lhs, const Literal& rhs) {
 	return lhs.index() < rhs.index();
 }
 
-//! returns !(lhs == rhs)
+//! Returns !(lhs == rhs).
 inline bool operator!=(const Literal& lhs, const Literal& rhs) {
 	return ! (lhs == rhs);
 }
@@ -188,24 +188,25 @@ inline void swap(Literal& l, Literal& r) {
 	l.swap(r);
 }
 
-typedef PodVector<Var>::type VarVec;          /**< a vector of variables  */
-typedef PodVector<Literal>::type LitVec;      /**< a vector of literals   */
-typedef PodVector<weight_t>::type WeightVec;  /**< a vector of weights    */
+typedef PodVector<Var>::type VarVec;          /**< A vector of variables.  */
+typedef PodVector<Literal>::type LitVec;      /**< A vector of literals.   */
+typedef PodVector<weight_t>::type WeightVec;  /**< A vector of weights.    */
+typedef PodVector<wsum_t>::type SumVec;       /**< A vector of sums of weights. */
 
-typedef std::pair<Literal, weight_t> WeightLiteral;  /**< a weight-literal */
-typedef PodVector<WeightLiteral>::type WeightLitVec; /**< a vector of weight-literals */
+typedef std::pair<Literal, weight_t> WeightLiteral;  /**< A weight-literal. */
+typedef PodVector<WeightLiteral>::type WeightLitVec; /**< A vector of weight-literals. */
 
 ///////////////////////////////////////////////////////////////////////////////
 // Truth values
 ///////////////////////////////////////////////////////////////////////////////
-typedef uint8 ValueRep;           /**< Type of the three value-literals */
-const ValueRep value_true   = 1;  /**< Value used for variables that are true */
-const ValueRep value_false  = 2;  /**< Value used for variables that are false */
-const ValueRep value_free   = 0;  /**< Value used for variables that are unassigned */
+typedef uint8 ValueRep;           /**< Type of the three value-literals. */
+const ValueRep value_true   = 1;  /**< Value used for variables that are true. */
+const ValueRep value_false  = 2;  /**< Value used for variables that are false. */
+const ValueRep value_free   = 0;  /**< Value used for variables that are unassigned. */
 
-//! returns the value that makes the literal lit true.
+//! Returns the value that makes the literal lit true.
 /*!
- * \param lit the literal for which the true-value should be determined.
+ * \param lit The literal for which the true-value should be determined.
  * \return
  *   - value_true     iff lit is a positive literal
  *   - value_false    iff lit is a negative literal.
@@ -213,18 +214,26 @@ const ValueRep value_free   = 0;  /**< Value used for variables that are unassig
  */
 inline ValueRep trueValue(const Literal& lit) { return 1 + lit.sign(); }
 
-//! returns the value that makes the literal lit false.
+//! Returns the value that makes the literal lit false.
 /*!
- * \param lit the literal for which the false-value should be determined.
+ * \param lit The literal for which the false-value should be determined.
  * \return
  *   - value_false      iff lit is a positive literal
  *   - value_true       iff lit is a negative literal.
  *   .
  */
 inline ValueRep falseValue(const Literal& lit) { return 1 + !lit.sign(); }
+
+//! Returns the sign that matches the value.
+/*!
+ * \return
+ *   - false iff v == value_true
+ *   - true  otherwise
+ */
+inline bool valSign(ValueRep v) { return v != value_true; }
 //@}
 
-//! Symbol table that maps external ids to internal literals
+//! Symbol table that maps external ids to internal literals.
 /*!
  * A symbol table can be populated incrementally, but
  * each incremental step must be started with a call to
@@ -260,7 +269,7 @@ public:
 	typedef map_type::const_iterator         const_iterator;
 	enum MapType { map_direct, map_indirect };
 
-	//! creates an empty symbol table
+	//! Creates an empty symbol table.
 	SymbolTable() : lastSort_(map_type::size_type(-1)), lastStart_(0), end_(varMax), type_(map_indirect) { }
 	~SymbolTable() { clear();  }
 	void   copyTo(SymbolTable& o) {
@@ -272,22 +281,22 @@ public:
 		o.lastSort_ = lastSort_;
 		o.lastStart_= lastStart_;
 	}
-	//! Type of symbol mapping
+	//! Type of symbol mapping.
 	/*!
-	 * In a direct mapping symbol ids directly correspond to solver variables.
-	 * In an indirect mapping symbol ids are independent of solver variables and
+	 * In a direct mapping, symbol ids directly correspond to solver variables.
+	 * In an indirect mapping, symbol ids are independent of solver variables and
 	 * are instead mapped to solver literals.
 	 */
 	MapType type()    const { return type_; }
-	//! returns the number of symbols in this table
+	//! Returns the number of symbols in this table.
 	uint32  size()    const { return type() == map_indirect ? (uint32)map_.size() : end_; }
-	//! returns an iterator pointing to the beginning of this index
+	//! Returns an iterator pointing to the beginning of this index.
 	const_iterator begin()    const { return map_.begin(); }
-	//! returns an iterator pointing to the first atom of the current incremental step
+	//! Returns an iterator pointing to the first atom of the current incremental step.
 	const_iterator curBegin() const { return map_.begin() + lastStart_; }
-	//! returns an iterator pointing behind the end of this index 
+	//! Returns an iterator pointing behind the end of this index.
 	const_iterator end()      const { return map_.end();   }
-	//! removes all symbols from this table
+	//! Removes all symbols from this table.
 	void   clear()      { 
 		for (const_iterator it = map_.begin(), end = map_.end(); it != end; ++it) {
 			freeName(it->second.name.c_str());
@@ -296,12 +305,12 @@ public:
 		lastSort_ = map_type::size_type(-1); 
 		lastStart_= 0; 
 	}
-	//! prepares the symbol table so that new symbols can be added
+	//! Prepares the symbol table so that new symbols can be added.
 	void startInit() {
 		lastStart_ = map_.size();
 		lastSort_  = lastStart_;
 	}
-	//! adds the symbol with the given id to the symbol table
+	//! Adds the symbol with the given id to the symbol table.
 	/*!
 	 * \pre startInit() was called
 	 * \pre the symbol table does not yet contain a symbol with the given id
@@ -311,10 +320,10 @@ public:
 		map_.push_back(value_type(id, symbol_type(negLit(0), dupName(name))));
 		return map_.back().second;
 	}
-	//! freezes the symbol table and prepares it so that lookup operations become valid
+	//! Freezes the symbol table and prepares it so that lookup operations become valid.
 	/*
-	 * \param type type of mapping
-	 * \param end  only for direct mapping: end of mapped range (i.e. [0, end))
+	 * \param type Type of mapping.
+	 * \param end  Only for direct mapping: end of mapped range (i.e. [0, end)).
 	 */
 	void endInit(MapType type = map_indirect, Var end = varMax) {
 		std::sort(map_.begin()+lastSort_, map_.end(), LessKey());
@@ -325,7 +334,7 @@ public:
 			end_    = end;
 		}
 	}
-	//! returns the symbol with id i or 0 if no such symbol exists
+	//! Returns the symbol with id i or 0 if no such symbol exists.
 	/*!
 	 * \pre endInit() was called
 	 */
@@ -334,7 +343,7 @@ public:
 		const_iterator it = std::lower_bound(map_.begin(), map_.end(), i, LessKey());
 		return it != map_.end() && it->first == i ? &it->second : 0;
 	}
-	//! returns the symbol with the given id
+	//! Returns the symbol with the given id.
 	/*!
 	 * \pre find(id) != 0
 	 */
