@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2009, Benjamin Kaufmann
+// Copyright (c) 2009, 2012 Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -39,7 +39,7 @@ namespace Clasp {
 struct VarScore {
 	VarScore() { clear(); }
 	void clear() { std::memset(this, 0, sizeof(VarScore)); }
-	//! Mark literal p as dependent
+	//! Mark literal p as dependent.
 	void setSeen( Literal p )    { seen_ |= uint32(p.sign()) + 1; }
 	//! Is literal p dependent?
 	bool seen(Literal p) const   { return (seen_ & (uint32(p.sign())+1)) != 0; }
@@ -54,14 +54,14 @@ struct VarScore {
 	//! Were both literals of this var tested?
 	bool testedBoth()      const { return tested_  == 3; }
 
-	//! Sets the score for literal p to value and marks p as tested
+	//! Sets the score for literal p to value and marks p as tested.
 	void setScore(Literal p, LitVec::size_type value) {
 		if (value > (1U<<14)-1) value = (1U<<14)-1;
 		if (p.sign()) nVal_ = uint32(value);
 		else          pVal_ = uint32(value);
 		setTested(p);
 	}
-	//! Sets the score of a dependent literal p to min(sc, current score)
+	//! Sets the score of a dependent literal p to min(sc, current score).
 	void setDepScore(Literal p, uint32 sc) {
 		if (!seen(p) || score(p) > sc) {
 			if (sc > (1U<<14)-1) sc = (1U<<14)-1;
@@ -73,8 +73,8 @@ struct VarScore {
 	uint32 score(Literal p) const { return p.sign() ? nVal_ : pVal_; }
 	//! Returns the scores of the two literals of a variable.
 	/*!
-	 * \param[out] mx the maximum score
-	 * \param[out] mn the minimum score
+	 * \param[out] mx The maximum score.
+	 * \param[out] mn The minimum score.
 	 */
 	void score(uint32& mx, uint32& mn) const {
 		if (nVal_ > pVal_) {
@@ -86,7 +86,7 @@ struct VarScore {
 			mn = nVal_;
 		}
 	}
-	//! returns the sign of the literal that has the higher score.
+	//! Returns the sign of the literal that has the higher score.
 	bool prefSign() const { return nVal_ > pVal_; }
 
 	uint32 nVal() const { return nVal_; }
@@ -98,7 +98,7 @@ private:
 	uint32 tested_: 2;
 };
 
-//! A small helper class used to score the result of a lookahead operation
+//! A small helper class used to score the result of a lookahead operation.
 struct ScoreLook {
 	enum Mode { score_max, score_max_min };
 	typedef PodVector<VarScore>::type VarScores; /**< A vector of variable-scores */
@@ -138,32 +138,32 @@ class Lookahead : public PostPropagator {
 public:
 	//! Defines the supported lookahead-types
 	enum Type { 
+		no_lookahead=0,   /**< Dummy value */
 		atom_lookahead,   /**< Test only atoms in both phases */
 		body_lookahead,   /**< Test only bodies in both phases */
 		hybrid_lookahead, /**< Test atoms and bodies but only their preferred decision literal */
-		no_lookahead      /**< Dummy value */
 	};
 	/*!
-	 * \param t Lookahead-type to use
+	 * \param t Lookahead-type to use.
 	 */
 	explicit Lookahead(Type t, bool topLevelImps = true);
 	~Lookahead();
 	
 	bool    init(Solver& s);
-	//! clears the lookahead list
+	//! Clears the lookahead list.
 	void    clear();
-	//! returns true if lookahead list is empty
+	//! Returns true if lookahead list is empty.
 	bool    empty() const { return head()->next == head_id; }
-	//! adds literal p to the lookahead list
+	//! Adds literal p to the lookahead list.
 	void    append(Literal p, bool testBoth);
-	//! single-step lookahead on all vars in the loookahead list
+	//! Single-step lookahead on all vars in the loookahead list.
 	bool    propagateFixpoint(Solver& s);
-	//! returns PostPropagator::priority_lookahead
+	//! Returns PostPropagator::priority_lookahead.
 	uint32  priority() const;
 	void    destroy(Solver* s, bool detach);
-	//! updates state with lookahead result 
+	//! Updates state with lookahead result.
 	ScoreLook score;
-	//! returns "best" literal w.r.t scoring of last lookahead
+	//! Returns "best" literal w.r.t scoring of last lookahead or posLit(0) if no such literal exists.
 	Literal heuristic(Solver& s);
 protected:
 	friend class UnitHeuristic;
@@ -192,7 +192,6 @@ private:
 	NodeId     last_;  // last candidate in list; invariant: node(last_)->next == head_id;
 	NodeId     pos_;   // current lookahead start position
 	uint32     top_;   // size of top-level
-	Literal    tlc_;   // top level choice
 };
 
 //! Heuristic that uses the results of lookahead.
@@ -213,30 +212,25 @@ private:
 class UnitHeuristic : public DecisionHeuristic {
 public:
 	/*!
-	 * \param t Lookahead-type to use
-	 * \param nant Score only atoms in NAnt(P)?
+	 * \param t Lookahead-type to use.
 	 */
-	explicit UnitHeuristic(Lookahead::Type t, bool nant = false);
+	explicit UnitHeuristic(Lookahead::Type t);
 	~UnitHeuristic();
 	void endInit(Solver& s);
-	void reinit(bool b) { init_ = b; }
 	void resurrect(const Solver&, Var v);
-	Literal doSelect(Solver& s) { return look_->heuristic(s); }
+	Literal doSelect(Solver& s);
 private:
 	Lookahead*      look_; // lookahead propagator
 	Lookahead::Type type_; // type of lookahead & heuristic
-	bool            init_; // used in incremental setting
-	bool            nant_; // score only atoms in NAnt(P)?
 };
 
 class RestrictedUnit : public DecisionHeuristic {
 public:
 	~RestrictedUnit();
-	//! extends the decision heuristic of s so that the next k choices are based on lookahead
+	//! Extends the decision heuristic of s so that the next k choices are based on lookahead.
 	static  void decorate(Solver& s, uint32 k, Lookahead* look);
 private:
 	explicit RestrictedUnit(uint32 k, Lookahead* look, DecisionHeuristic* def);
-	void reinit(bool b)                    { default_->reinit(b);    }
 	void startInit(const Solver& s)        { default_->startInit(s); }
 	void resurrect(const Solver& s, Var v) { default_->resurrect(s, v); }
 	void simplify(const Solver&  s, LitVec::size_type st) { default_->simplify(s, st); }
