@@ -36,13 +36,13 @@ namespace Clasp {
 class ModelEnumerator : public Enumerator {
 public:
 	/*! 
-	 * \param p the printer to use for outputting results.
+	 * \param p The printer to use for outputting results.
 	 */
 	explicit ModelEnumerator(Enumerator::Report* p = 0);
 	~ModelEnumerator();
-	//! enables/disbales projective solution enumeration
+	//! Enables/disbales projective solution enumeration.
 	/*!
-	 * \note must be called before Enumerator::init() is called
+	 * \note Must be called before Enumerator::init() is called.
 	 */
 	void     setEnableProjection(bool b);
 protected:
@@ -80,6 +80,7 @@ public:
 	 */
 	BacktrackEnumerator(uint32 projectOpts, Enumerator::Report* p = 0);
 	bool supportsRestarts() const { return optimize(); }
+	bool supportsParallel() const { return !projectionEnabled(); }
 private:
 	enum ProjectOptions {
 		ENABLE_HEURISTIC_SELECT = 1,
