@@ -69,7 +69,7 @@ struct ParallelSolveOptions {
 	 */
 	void createSolveObject(SolveAlgorithm*& out, SharedContext& ctx, SolverConfig** sc) const;
 	//! Returns the number of threads that can run concurrently on the current hardware.
-	static uint32   recommendedSolvers() { return std::thread::hardware_concurrency(); }
+	static uint32   recommendedSolvers() { return Clasp::thread::hardware_concurrency(); }
 	//! Returns number of maximal number of supported threads.
 	static uint32   supportedSolvers()   { return 64; }
 };
@@ -179,7 +179,7 @@ public:
 	int  error() const   { return (int)error_; }
 	void setWinner()     { win_ = 1; }
 	bool winner() const  { return win_ != 0; }
-	void setThread(std::thread& x) { assert(!joinable()); x.swap(thread_); assert(joinable()); }
+	void setThread(Clasp::thread& x) { assert(!joinable()); x.swap(thread_); assert(joinable()); }
 	
 	//! True if *this has an associated thread of execution, false otherwise.
 	bool joinable() const { return thread_.joinable(); }
@@ -255,7 +255,7 @@ private:
 	ParallelSolve* ctrl() const { return messageHandler_.ctrl; }
 	typedef LitVec::size_type size_type;
 	typedef PodVector<Constraint*>::type ClauseDB;
-	std::thread        thread_;     // active thread or empty for master
+	Clasp::thread      thread_;     // active thread or empty for master
 	ClauseDB           integrated_; // my integrated clauses
 	Solver*            solver_;     // my solver
 	const SolveParams* params_;     // my solving params
