@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2007, Benjamin Kaufmann
+// Copyright (c) 2006, 2007, 2012 Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -60,7 +60,7 @@ const ValueRep value_weak_true = 3; /**< true but no proof */
  */
 //@{
 
-//! A list of variables and their types
+//! A list of variables and their types.
 /*!
  * A VarList stores the type of each variable, which allows for easy variable-cloning.
  * It also stores flags for each literal to be used in "mark and test"-algorithms.
@@ -68,23 +68,23 @@ const ValueRep value_weak_true = 3; /**< true but no proof */
 class VarList {
 public:
 	VarList() { add(Var_t::atom_var); }
-	//! adds a new var of type t to the list
+	//! Adds a new var of type t to the list.
 	Var   add(VarType t) {
 		vars_.push_back( static_cast<uint8>(t << 5) );
 		return (Var)vars_.size()-1;
 	}
-	//! marks v as equivalent to both a body and an atom
+	//! Marks v as equivalent to both a body and an atom.
 	void  setAtomBody(Var v) { setFlag(v, eq_f); }
-	//! adds to s all variables contained in this var list that are >= startVar 
+	//! Adds to s all variables contained in this var list that are >= startVar.
 	void  addTo(SharedContext& ctx, Var startVar);
-	//! returns true if this var list is empty
+	//! Returns true if this var list is empty.
 	bool empty() const { return vars_.size() == 1; }
-	//! returns the number of vars contained in this list
+	//! Returns the number of vars contained in this list.
 	VarVec::size_type size() const { return vars_.size(); }
-	//! clears the var list
+	//! Clears the var list.
 	void clear() { Vars().swap(vars_); add(Var_t::atom_var); }
 	
-	//! removes all vars > startVar
+	//! Removes all vars > startVar.
 	void shrink(Var startVar) {
 		startVar = std::max(Var(1), startVar);
 		vars_.resize(startVar);
@@ -117,7 +117,7 @@ private:
 	Vars          vars_;
 };
 
-//! Program statistics for *one* incremental step
+//! Program statistics for *one* incremental step.
 struct PreproStats {
 	PreproStats() : trStats(0) { reset(); }
 	~PreproStats()             { delete trStats; }
@@ -187,7 +187,7 @@ public:
 	typedef SharedDependencyGraph PBADG;
 	typedef PBADG*                GraphPtr;
 
-	//! Defines the possible modes for handling extended rules, i.e. choice, cardinality, and weight rules
+	//! Defines the possible modes for handling extended rules, i.e. choice, cardinality, and weight rules.
 	enum ExtendedRuleMode {
 		mode_native           = 0, /**< Handle extended rules natively                          */
 		mode_transform        = 1, /**< Transform extended rules to normal rules                */
@@ -198,16 +198,16 @@ public:
 		mode_transform_dynamic= 6  /**< Heuristically decide whether or not to transform a particular extended rule */
 	};
 
-	//! Options for the (Eq)-Preprocessor
+	//! Options for the (Eq)-Preprocessor.
 	struct EqOptions {
 		EqOptions() : iters(5), erMode(mode_native), noSCC(false), dfOrder(false), backprop(false), normalize(false) {}
-		EqOptions& iterations(int it)      { iters   = it;   return *this;}
+		EqOptions& iterations(uint32 it)   { iters   = it;   return *this;}
 		EqOptions& depthFirst()            { dfOrder = true; return *this;}
 		EqOptions& backpropagate()         { backprop= true; return *this;}
 		EqOptions& noScc()                 { noSCC   = true; return *this;}
 		EqOptions& noEq()                  { iters   = 0;    return *this;}
 		EqOptions& ext(ExtendedRuleMode m) { erMode = m;     return *this;}
-		int32            iters;    /**< Number of iterations - 0 = disabled */
+		uint32           iters;    /**< Number of iterations - 0 = disabled */
 		ExtendedRuleMode erMode;   /**< ExtendedRuleMode */
 		bool             noSCC;    /**< disable scc checking, i.e. only compute supported models? */  
 		bool             dfOrder;  /**< Classify in depth-first order? */
@@ -317,7 +317,7 @@ public:
 	 *   - The atom was not yet added to the symbol table, i.e. 
 	 *     setAtomName() is called at most once for an atom.
 	 *   . 
-	 * \param atomId The id of the atom for which a name should be set
+	 * \param atomId The id of the atom for which a name should be set.
 	 * \param name The new name of the atom with the given id.
 	 * \note If atomId is not yet known, an atom with the given id is implicitly created. 
 	 *
@@ -598,29 +598,29 @@ class PrgNode {
 public:
 	static const uint32 noScc = (1U << 28) - 1;
 	
-	//! creates a new node
+	//! Creates a new node.
 	/*!
-	 * \note The ctor creates a node that corresponds to a literal that is false
+	 * \note The ctor creates a node that corresponds to a literal that is false.
 	 */
 	explicit PrgNode(Literal lit = negLit(sentVar)) 
 		: lit_(lit), root_(0), value_(value_free), scc_(noScc), eq_(0), seen_(0), ignore_(0), frozen_(0) {}
 	
-	//! returns true if node has an associated variable in a solver
+	//! Returns true if node has an associated variable in a solver.
 	/*!
 	 * If hasVar() returns false, the node is not relevant and must not be used any further.
 	 */
 	bool    hasVar()    const { return lit_ != negLit(sentVar); }
-	//! returns the variable associated with this node or sentVar if no var is associated with this node
+	//! Returns the variable associated with this node or sentVar if no var is associated with this node.
 	Var     var()       const { return lit_.var(); }
-	//! returns the literal associated with this node or a sentinel literal if no var is associated with this node
+	//! Returns the literal associated with this node or a sentinel literal if no var is associated with this node.
 	Literal literal()   const { return lit_; }
 
-	//! returns the value currently assigned to this node
+	//! Returns the value currently assigned to this node.
 	ValueRep  value()     const { return value_; }
 	
-	//! returns the literal that must be true in order to fulfill the truth-value of this node
+	//! Returns the literal that must be true in order to fulfill the truth-value of this node.
 	/*!
-	 * \note if value() == value_free, the special sentinel literal that is true
+	 * \note If value() == value_free, the special sentinel literal that is true
 	 * in every solver is returned.
 	 *
 	 * \return
@@ -633,10 +633,10 @@ public:
 		return Literal();
 	}
 	
-	//! returns the node's component number or PrgNode::noScc if node is trivially connected or sccs where not yet identified
+	//! Returns the node's component number or PrgNode::noScc if node is trivially connected or sccs where not yet identified.
 	uint32    scc()       const { return scc_; }
 	
-	//! returns true if this node is equivalent to some other node
+	//! Returns true if this node is equivalent to some other node.
 	/*!
 	 * If eq() is true, the node is no longer relevant and must not be used any further.
 	 * The only sensible operation is to call root() in order to get the id of this node's
@@ -644,22 +644,22 @@ public:
 	 */
 	bool      eq()        const { return eq_ != 0; }
 	
-	//! returns true if this node should be ignored during SCC checking
+	//! Returns true if this node should be ignored during SCC checking.
 	/*!
-	 * \note for bodies only: if ignore() is true, the body is known to be false and no longer relevant
+	 * \note For bodies only: if ignore() is true, the body is known to be false and no longer relevant.
 	 */
 	bool      ignore()    const { return ignore_ != 0; }
 
-	//! returns true if this atom is frozen, i.e. to be defined in a later step
+	//! Returns true if this atom is frozen, i.e. to be defined in a later step.
 	bool      frozen()    const { return frozen_ != 0; }
 
-	//! returns the id of the node to which this node is equivalent
+	//! Returns the id of the node to which this node is equivalent.
 	/*! 
 	 * \pre eq() is true
 	 */
 	Var       eqNode()    const { assert(eq()); return root_; }
 	
-	//! returns the id of this node in the unfounded set checker
+	//! Returns the id of this node in the unfounded set checker.
 	/*! 
 	 * \pre eq() is false
 	 */
@@ -703,7 +703,7 @@ public:
 	
 	/*!
 	 * \name SCC checking
-	 * Intended to be used only by the SCC checker
+	 * Intended to be used only by the SCC checker.
 	 */
 	//@{
 	uint32    dfsIdx()    const  { return root_;  }
@@ -760,9 +760,9 @@ private:
 struct HeadEdge {
 	enum Type { NORMAL_HEAD = 0, UNUSED_HEAD_T1 = 1, UNUSED_HEAD_T2 = 2, CHOICE_HEAD = 3};
 	explicit HeadEdge(uint32 nId = 0, Type t = NORMAL_HEAD) : rep( (nId << 2) | t ) {}
-	//! returns the id of the adjacent node
+	//! Returns the id of the adjacent node.
 	uint32 node()   const { return rep >> 2; }
-	//! the type of this edge
+	//! Rhe type of this edge.
 	Type   type()   const { return Type(rep & 3u); }
 	bool   choice() const { return type() == CHOICE_HEAD; }
 	bool   normal() const { return type() == NORMAL_HEAD; }
@@ -780,16 +780,16 @@ class PrgBodyNode : public PrgNode {
 public:
 	enum BodyType { NORMAL_BODY = 0, COUNT_BODY = 1, SUM_BODY = 2};
 
-	//! creates a new body node and connects the node to its predecessors
+	//! Creates a new body node and connects the node to its predecessors.
 	/*!
-	 * \param id      The id of the new body object
-	 * \param rule    The rule for which this body object is created
-	 * \param rInfo   The rule's simplification object
-	 * \param prg     The program in which the new body is used
+	 * \param id      The id of the new body object.
+	 * \param rule    The rule for which this body object is created.
+	 * \param rInfo   The rule's simplification object.
+	 * \param prg     The program in which the new body is used.
 	 */
 	static PrgBodyNode* create(uint32 id, const PrgRule& rule, const PrgRule::RData& rInfo, ProgramBuilder& prg);
 	
-	//! destroys a body node created via create(uint32 id, const PrgRule& rule, const PrgRule::RData& rInfo, ProgramBuilder& prg)
+	//! Destroys a body node created via create(uint32 id, const PrgRule& rule, const PrgRule::RData& rInfo, ProgramBuilder& prg).
 	void destroy();
 	
 	BodyType type() const { return BodyType(type_); }
@@ -809,7 +809,7 @@ public:
 		}
 	}
 
-	//! returns true if the body node is supported.
+	//! Returns true if the body node is supported.
 	/*!
 	 * A normal body is supported, iff all of its positive subgoals are supported.
 	 * A count/sum body is supported if the sum of the weights of the supported positive +
@@ -817,7 +817,7 @@ public:
 	 */
 	bool isSupported() const { return !extended() ? extra_.unsupp <= 0 : extra_.ext->unsupp <= 0; }
 
-	//! notifies the body node about the fact that its positive subgoal v is supported
+	//! Notifies the body node about the fact that its positive subgoal v is supported.
 	/*!
 	 * \return
 	 *  - true if the body is now also supported
@@ -826,19 +826,19 @@ public:
 	 */
 	bool onPosPredSupported(Var /* v */);
 	
-	//! simplifies the body, i.e. its predecessors-lists
+	//! Simplifies the body, i.e. its predecessors-lists.
 	/*!
 	 * - removes true/false atoms from B+/B- resp.
 	 * - removes/merges duplicate subgoals
 	 * - checks whether body must be false (e.g. contains false/true atoms in B+/B- resp. or contains p and ~p)
 	 * - computes a new hash value
 	 *
-	 * \param prg The program in which this body is used
-	 * \param bodyId The body's id in the program
+	 * \param prg The program in which this body is used.
+	 * \param bodyId The body's id in the program.
 	 * \param[out] a pair of hash-values. hashes.first is the old hash value. hash.second the new hash value.
-	 * \param pre The preprocessor that changed this body
+	 * \param pre The preprocessor that changed this body.
 	 * \param strong If true, treats atoms that have no variable associated as false. Otherwise
-	 *               such atoms are ignored during simplification
+	 *               such atoms are ignored during simplification.
 	 * \note If body must be false, calls setValue(value_false) and sets the
 	 * ignore flag to true.
 	 * \return
@@ -847,7 +847,7 @@ public:
 	 */
 	bool simplifyBody(ProgramBuilder& prg, uint32 bodyId, std::pair<uint32, uint32>& hashes, Preprocessor& pre, bool strong);
 	
-	//! simplifies the heads of this body
+	//! Simplifies the heads of this body.
 	/*!
 	 * Removes superfluous heads and sets the body to false if for some atom a
 	 * in the head of this body B, Ta -> FB. In that case, all heads atoms are removed, because
@@ -860,82 +860,82 @@ public:
 	 */
 	bool simplifyHeads(ProgramBuilder& prg, Preprocessor& pre, bool strong);
 
-	//! sets the body to false and removes it from its heads
+	//! Sets the body to false and removes it from its heads.
 	bool propagateFalse(uint32 bodyId, AtomList& atoms);
-	//! backpropagates the truth-value of this body to its subgoals
+	//! Backpropagates the truth-value of this body to its subgoals.
 	bool backpropagate(ProgramBuilder& prg, LitVec& compute);
-	//! replaces *this with other in all rules
+	//! Replaces *this with other in all rules.
 	/*!
 	 * \return number of new NORMAL_HEAD-edges added to other
 	 */
 	uint32 replace(PrgBodyNode& other, uint32 otherId, Preprocessor& pre);
 
-	//! adds the body-oriented nogoods as a set of constraints to the solver.
+	//! Adds the body-oriented nogoods as a set of constraints to the solver.
 	/*
 	 * \return false on conflict
 	 */
 	bool toConstraint(SharedContext&, ClauseCreator& c, const ProgramBuilder& prg);
 	
-	//! returns the bound of this body
+	//! Returns the bound of this body.
 	/*!
 	 * \note The bound of a normal body is equal to the size of the body
 	 */
 	weight_t bound() const;
 
-	//! returns the sum of the subgoals weights
+	//! Returns the sum of the subgoals weights.
 	/*!
 	 * \note The sum of a normal body is equal to the size of the body
 	 */
 	weight_t sumWeights() const;
 
-	//! returns the number of atoms in the body
+	//! Returns the number of atoms in the body.
 	uint32 size()    const { return size_; }
-	//! returns the number of atoms in the positive body (B+)
+	//! Returns the number of atoms in the positive body (B+).
 	uint32 posSize() const { return posSize_; }
-	//! returns the number of atoms in the negative body (B-)
+	//! Returns the number of atoms in the negative body (B-).
 	uint32 negSize() const { return size() - posSize(); }
 
-	//! returns the idx'th positive subgoal
+	//! Returns the idx'th positive subgoal.
 	/*! 
 	 * \pre idx < posSize()
 	 * \note first positive subgoal has index 0
 	 */
 	Var pos(uint32 idx) const { assert(idx < posSize()); return goals_[idx].var(); }
 
-	//! returns the idx'th negative subgoal
+	//! Returns the idx'th negative subgoal.
 	/*! 
 	 * \pre idx < negSize()
 	 * \note first negative subgoal has index 0
 	 */
 	Var neg(uint32 idx) const { assert(idx < negSize()); return goals_[posSize_+idx].var(); }
 
-	//! returns the idx'th subgoal as a literal
+	//! Returns the idx'th subgoal as a literal.
 	Literal goal(uint32 idx) const { 
 		assert(idx < size()); return goals_[idx];
 	}
 
-	//! returns the weight of the specified subgoal.
+	//! Returns the weight of the specified subgoal.
 	/*!
-	 * \param idx the index of the subgoal
-	 * \param pos true for a positive, false for a negative subgoal
+	 * \param idx The index of the subgoal.
+	 * \param pos true for a positive, false for a negative subgoal.
 	 */
 	weight_t weight(uint32 /* idx */, bool /* pos */) const;
 
-	//! returns the weight of the specified subgoal.
+	//! Returns the weight of the specified subgoal.
 	weight_t  weight(uint32 idx) const { return !hasWeights() ? 1 : extra_.ext->weights[idx]; }
 	
-	//! adds a rule edge between this body and the given atom
+	//! Adds a rule edge between this body and the given atom.
 	HeadType addHead(Var atomId, RuleType t) {
 		heads_.push_back(HeadEdge(atomId, t != CHOICERULE ? HeadEdge::NORMAL_HEAD : HeadEdge::CHOICE_HEAD));
 		return heads_.back().type();
 	}
-	//! removes all rule edges between this body and the given atom
+	//! Removes all rule edges between this body and the given atom.
 	void removeHead(Var atomId);
-	//! returns true if this body defines any atom
+	//! Returns true if this body defines any atom.
 	bool hasHeads() const { return !heads_.empty(); }
-	//! establishes set property for the heads of this body
+	//! Establishes set property for the heads of this body.
 	void buildHeadSet();
-	//! returns true if there is a rule edge between atom x and this body
+	//! Returns true if there is a rule edge between atom x and this body.
 	/*!
 	 * \pre buildHeadSet() was called
 	 **/
@@ -954,7 +954,7 @@ public:
 			default:             return NORMAL_BODY;
 		}
 	}
-	//! sorts B+ and B- by increasing atom ids
+	//! Sorts B+ and B- by increasing atom ids.
 	void sortBody();
 	uint32 reinitDeps(uint32 id, ProgramBuilder& prg);
 private:
@@ -1002,23 +1002,23 @@ private:
 	Literal   goals_[0];      // B+: [0, posSize_), B-: [posSize_, size_)
 };
 
-//! An atom-node in a body-atom-dependency graph
+//! An atom-node in a body-atom-dependency graph.
 class PrgAtomNode : public PrgNode {
 public:
-	//! adds the atom-oriented nogoods to as a set of constraints to the solver.
+	//! Adds the atom-oriented nogoods to as a set of constraints to the solver.
 	/*
 	 * \return false on conflict
 	 */
 	bool toConstraint(SharedContext&, ClauseCreator& c, ProgramBuilder& prg);
 	
-	//! simplifies this atom, i.e. its predecessors-list
+	//! Simplifies this atom, i.e. its predecessors-list.
 	/*!
 	 * - removes false/irrelevant bodies
 	 *
-	 * \param atomId The id of this atom
-	 * \param prg The program in which this atom is defined
-	 * \param pre The preprocessor that changed this atom or its predecessors
-	 * \param strong If true, updates bodies that were replaced with equivalent bodies
+	 * \param atomId The id of this atom.
+	 * \param prg The program in which this atom is defined.
+	 * \param pre The preprocessor that changed this atom or its predecessors.
+	 * \param strong If true, updates bodies that were replaced with equivalent bodies.
 	 * \note If atom must be false, calls setValue(value_false) and sets the
 	 * ignore flag to true.
 	 * \return
