@@ -200,7 +200,6 @@ private:
 class PrgRuleTransform {
 public:
 	PrgRuleTransform();
-	~PrgRuleTransform();
 	uint32 transform(ProgramBuilder& prg, PrgRule& rule);
 	uint32 transformNoAux(ProgramBuilder& prg, PrgRule& rule);
 	static weight_t prepareRule(PrgRule& rule, weight_t* sumVec);
@@ -209,7 +208,6 @@ private:
 	PrgRuleTransform& operator=(const PrgRuleTransform&);
 	uint32 transformChoiceRule(ProgramBuilder& prg, PrgRule& rule) const;
 	class Impl;
-	Impl* impl_;
 };
 
 }
