@@ -249,7 +249,7 @@ public:
 	 * Shall return true if termination is supported, otherwise false.
 	 */
 	virtual bool   terminate() = 0;
-	virtual bool   hasErrors() { return false; }
+	virtual bool   hasErrors() const { return false; }
 	//! Runs the solve algorithm.
 	/*!
 	 * \param ctx    A fully initialized context object containing the problem.
