@@ -44,53 +44,57 @@ class SharedLiterals;
 ///////////////////////////////////////////////////////////////////////////////
 // Statistics
 ///////////////////////////////////////////////////////////////////////////////
-//! A struct for aggregating statistics relevant for parallel solving.
-/*!
- * Always associated with one solver (thread).
- */
-struct ParallelStats {
-	ParallelStats() { reset(); }
-	double  cpuTime;  /**< (Estimated) cpu time of the current solver. */
-	uint64  splits;   /**< Number of split requests handled. */
-	uint64  gps :62;  /**< Number of guiding paths received. */
-	uint64  term:1;   /**< Solver was terminated             */
-	uint64  win :1;   /**< Solver terminated the algorithm   */
-	uint64  gpLits;   /**< Sum of literals in received guiding paths. */
-	uint64  shared;   /**< Number of nogoods shared. */
-	uint64  imps;     /**< Number of initial implications from shared. */
-	uint64  jumps;    /**< Sum of backjumps needed to integrate new implications. */
-	void    reset() { 
-		std::memset(this, 0, sizeof(*this)); 
-		cpuTime    = 0.0;
+//! A struct for holding core solving statistics used by a solver.
+struct CoreStats {
+	CoreStats() { reset(); }
+	void reset(){ std::memset(this, 0, sizeof(*this)); }
+	void accu(const CoreStats& o) {
+		const uint64* rhs = &o.choices;
+		      uint64* lhs = &choices;
+		while (rhs != &o.cflLast) { *lhs++ += *rhs++; }
+		cflLast = std::max(o.cflLast, cflLast);
+		lhs     = learnts; rhs = o.learnts;
+		while (rhs <= &o.deleted) { *lhs++ += *rhs++; }
+		binary += o.binary;
+		ternary+= o.ternary;
 	}
-	void    accu(const ParallelStats& o) {
-		cpuTime  += o.cpuTime;
-		splits   += o.splits;
-		gps      += o.gps;
-		term     |= o.term;
-		win      |= o.win;
-		gpLits   += o.gpLits;
-		shared   += o.shared;
-		imps     += o.imps;
-		jumps    += o.jumps;
-	}
-	void newGP(LitVec::size_type length) {
-		++gps;
-		gpLits += length;
+	void addLearnt(uint32 size, ConstraintType t) {
+		assert(t != Constraint_t::static_constraint && t <= Constraint_t::max_value);
+		learnts[t-1]+= 1;
+		lits[t-1]   += size;
+		binary      += (size == 2);
+		ternary     += (size == 3);
 	}
+	double avgLbd() const { return sumLbd / (double)analyzed; }
+	double avgCfl() const { return sumCfl / (double)analyzed; }
+	uint64 choices;   /**< Number of choices performed. */
+	uint64 conflicts; /**< Number of conflicts found. */
+	uint64 analyzed;  /**< Number of backjumps (i.e. number of analyzed conflicts). */
+	uint64 restarts;  /**< Number of restarts. */ 
+	uint64 models;    /**< Number of models found. */
+	uint64 sumCfl;    /**< Sum of conflict levels. */
+	uint64 sumLbd;    /**< Sum of lbds.            */
+	uint64 cflLast;   /**< Number of conflicts since last restart. */
+	// CL2 - Learnt stats
+	typedef uint64 Array[Constraint_t::max_value];
+	Array  learnts;   /**< learnts[t-1]: Number of learnt nogoods of type t.  */
+	Array  lits;      /**< lits[t-1]   : Sum of literals in nogoods of type t.*/
+	uint64 deleted;   /**< Sum of learnt nogoods removed. */
+	uint32 binary;    /**< Number of learnt binary nogoods. */
+	uint32 ternary;   /**< Number of learnt ternary nogoods.*/
 };
-
 //! A struct for jump statistics.
 struct JumpStats {
 	JumpStats() { reset(); }
 	void reset(){ std::memset(this, 0, sizeof(*this)); }
 	void accu(const JumpStats& o) {
+		modLits += o.modLits;
 		bJumps  += o.bJumps;
 		jumpSum += o.jumpSum;
 		boundSum+= o.boundSum;
-		if (o.maxJump   > maxJump)   maxJump   = o.maxJump;
-		if (o.maxJumpEx > maxJumpEx) maxJumpEx = o.maxJumpEx;
-		if (o.maxBound  > maxBound)  maxBound  = o.maxBound;
+		maxJump  = std::max(o.maxJump, maxJump);
+		maxJumpEx= std::max(o.maxJumpEx, maxJumpEx);
+		maxBound = std::max(o.maxBound, maxBound);
 	}
 	void    update(uint32 dl, uint32 uipLevel, uint32 bLevel) {
 		jumpSum += dl - uipLevel; 
@@ -103,6 +107,7 @@ struct JumpStats {
 		}
 		else { maxJumpEx = maxJump; }
 	}
+	uint64  modLits;  /**< Sum of decision literals in models. */
 	uint64  bJumps;   /**< Number of backjumps that were bounded. */
 	uint64  jumpSum;  /**< Number of levels that could be skipped w.r.t first-uip. */
 	uint64  boundSum; /**< Number of levels that could not be skipped because of backtrack-level.*/
@@ -110,68 +115,42 @@ struct JumpStats {
 	uint32  maxJumpEx;/**< Longest executed backjump (< maxJump if longest jump was bounded). */
 	uint32  maxBound; /**< Max difference between uip- and backtrack-level. */
 };
-
-//! A struct for holding core solving statistics used by a solver.
-struct CoreStats {
-	CoreStats() { reset(); }
-	void reset(){ std::memset(this, 0, sizeof(*this)); }
-	void accu(const CoreStats& o) {
-		choices  += o.choices;
-		conflicts+= o.conflicts;
-		analyzed += o.analyzed;
-		restarts += o.restarts;
-		lRestart  = std::max(o.lRestart, lRestart);
-		models   += o.models;
-		sumCfl   += o.sumCfl;
-		sumLbd   += o.sumLbd;
-		
-		binary   += o.binary;
-		ternary  += o.ternary;
-		deleted  += o.deleted;
-		modLits  += o.modLits;
-		for (int i = 0; i != Constraint_t::max_value; ++i) {
-			learnts[i] += o.learnts[i];
-			lits[i]    += o.lits[i];
-		}
+//! A struct for aggregating statistics relevant for parallel solving.
+/*!
+ * Always associated with one solver (thread).
+ */
+struct ParallelStats {
+	ParallelStats() { reset(); }
+	double  cpuTime;    /**< (Estimated) cpu time of the current solver. */
+	uint64  distributed;/**< Number of nogoods distributed. */
+	uint64  integrated; /**< Number of nogoods integrated   */
+	uint64  sumLbd;     /**< Sum of lbds of shared nogoods. */
+	uint64  imps;       /**< Number of initial implications from shared. */
+	uint64  jumps;      /**< Sum of backjumps needed to integrate new implications. */
+	uint64  gpLits;     /**< Sum of literals in received guiding paths. */
+	uint32  gps;        /**< Number of guiding paths received. */
+	uint32  splits;     /**< Number of split requests handled. */
+	void    reset() { 
+		std::memset(this, 0, sizeof(*this)); 
+		cpuTime = 0.0;
 	}
-	void addLearnt(uint32 size, ConstraintType t) {
-		assert(t != Constraint_t::static_constraint && t <= Constraint_t::max_value);
-		++learnts[t-1];
-		lits[t-1] += size;
-		binary += (size == 2);
-		ternary+= (size == 3);
+	void    accu(const ParallelStats& o) {
+		cpuTime     += o.cpuTime;
+		// dist stats
+		distributed += o.distributed;
+		integrated  += o.integrated;
+		sumLbd      += o.sumLbd;
+		imps        += o.imps;
+		jumps       += o.jumps;
+		// gp stats
+		gpLits      += o.gpLits;
+		splits      += o.splits;
+		gps         += o.gps;
 	}
-	void removeLearnt(uint32 size, ConstraintType t) {
-		assert(t != Constraint_t::static_constraint && t <= Constraint_t::max_value);
-		--learnts[t-1];
-		lits[t-1] -= size;
-		binary -= (size == 2);
-		ternary-= (size == 3);
+	void newGP(LitVec::size_type length) {
+		++gps;
+		gpLits += length;
 	}
-	void addModel(uint32 size) {
-		++models;
-		modLits += size;
-	}
-	void removeModel(uint32 size) {
-		--models;
-		modLits -= size;
-	}
-	double avgLbd() const { return sumLbd / (double)analyzed; }
-	double avgCfl() const { return sumCfl / (double)analyzed; }
-	uint64 choices;   /**< Number of choices performed. */
-	uint64 conflicts; /**< Number of conflicts found. */
-	uint64 analyzed;  /**< Number of backjumps (i.e. number of analyzed conflicts). */
-	uint64 restarts;  /**< Number of restarts. */ 
-	uint64 lRestart;  /**< Number of conflicts on last restart. */
-	uint64 models;    /**< Number of models found. */
-	uint64 sumCfl;    /**< Sum of conflict levels. */
-	uint64 sumLbd;    /**< Sum of lbds.            */
-	uint64 learnts[Constraint_t::max_value]; /**< Number of learnt nogoods of type t-1. */
-	uint64 lits[Constraint_t::max_value];    /**< Sum of literals in nogoods of type t-1. */
-	uint32 binary;    /**< Number of learnt binary nogoods. */
-	uint32 ternary;   /**< Number of learnt ternary nogoods.*/
-	uint64 deleted;   /**< Sum of learnt nogoods removed. */
-	uint64 modLits;   /**< Sum of decision literals in models. */
 };
 
 struct QueueImpl {
@@ -219,89 +198,81 @@ private: SumQueue(uint32 size) : sumLbd(0), sumCfl(0), samples(0), queue(size) {
 private: SumQueue(const SumQueue&);
 private: SumQueue& operator=(const SumQueue&);
 };
-
 //! A struct for aggregating statistics of one solve operation.
 struct SolveStats : public CoreStats {
-	SolveStats() : jumps(0), queue(0), parallel(0) { }
-	SolveStats(const SolveStats& o) : CoreStats(o), jumps(0), queue(0), parallel(0) { 
+	SolveStats() : queue(0), jumps(0), parallel(0) {}
+	SolveStats(const SolveStats& o) : CoreStats(o), queue(0), jumps(0), parallel(0) {
+		if (o.queue)    enableQueue(o.queue->maxSize());
 		if (o.jumps)    jumps    = new JumpStats(*o.jumps); 
 		if (o.parallel) parallel = new ParallelStats(*o.parallel);
-		if (o.queue)    enableQueue(o.queue->maxSize());
 	}
-	~SolveStats() { delete jumps; delete parallel; if (queue) queue->destroy();   }
-	void enableJumpStats()        { if (!jumps)   jumps    = new JumpStats();     }
-	void enableParallelStats()    { if (!parallel)parallel = new ParallelStats(); }
-	void enableQueue(uint32 size) { 
-		if (queue && queue->maxSize()!=size) { queue->destroy(); queue = 0; }
-		if (!queue)                          { queue = SumQueue::create(size); }
-	}
-	void reset() {  
-		CoreStats::reset();
-		if (jumps)   jumps->reset();
-		if (queue)   queue->reset();
-		if (parallel)parallel->reset();
-	}
-	void accu(const SolveStats& o) {
-		CoreStats::accu(o);
-		if (jumps && o.jumps) jumps->accu(*o.jumps);
-		if (parallel && o.parallel) parallel->accu(*o.parallel);
-	}
-	void updateJumps(uint32 dl, uint32 uipLevel, uint32 bLevel, uint32 lbd) {
-		++analyzed;
-		sumCfl += dl;
-		sumLbd += lbd;
-		if (jumps) { jumps->update(dl, uipLevel, bLevel); }
-		if (queue) { queue->update(dl, lbd); }
-	}
-	JumpStats*     jumps;    /**< Optional jump statistics. */
+	~SolveStats() { delete parallel; delete jumps; if (queue) queue->destroy(); }
+	void enableParallelStats() { if (!parallel)parallel = new ParallelStats(); }
+	void enableStats(uint32 level);
+	void enableStats(const SolveStats& other);
+	void enableQueue(uint32 size);
+	void reset();
+	void accu(const SolveStats& o);
+	void swapStats(SolveStats& o);
+	inline void updateJumps(uint32 dl, uint32 uipLevel, uint32 bLevel, uint32 lbd);
+	inline void addDistributed(uint32 lbd, ConstraintType t);
+	inline void addIntegratedAsserting(uint32 receivedDL, uint32 jumpDL);
+	inline void addIntegrated(uint32 num = 1);
+	inline void removeIntegrated(uint32 num = 1);
+	inline void addDeleted(uint32 num);
+	inline void addModel(uint32 decisionLevel);
 	SumQueue*      queue;    /**< Optional queue for running averages. */
+	JumpStats*     jumps;    /**< Optional jump statistics. */
 	ParallelStats* parallel; /**< Optional parallel statistics. */
-private:
-	SolveStats& operator=(const SolveStats&);
+private: SolveStats& operator=(const SolveStats&);
 };
+inline void SolveStats::addDeleted(uint32 num)                    { deleted += num; }
+inline void SolveStats::addDistributed(uint32 lbd, ConstraintType){ if (parallel) { ++parallel->distributed; parallel->sumLbd += lbd; } }
+inline void SolveStats::updateJumps(uint32 dl, uint32 uipLevel, uint32 bLevel, uint32 lbd) {
+	++analyzed;
+	sumCfl += dl;
+	sumLbd += lbd;
+	if (queue) { queue->update(dl, lbd); }
+	if (jumps) { jumps->update(dl, uipLevel, bLevel); }
+}
+inline void SolveStats::addIntegratedAsserting(uint32 rDL, uint32 jDL) {
+	if (parallel) { ++parallel->imps; parallel->jumps += (rDL - jDL); }
+}
+inline void SolveStats::addIntegrated(uint32 n)   { if (parallel) { parallel->integrated += n;} }
+inline void SolveStats::removeIntegrated(uint32 n){ if (parallel) { parallel->integrated -= n;} }
+inline void SolveStats::addModel(uint32 DL) {
+	++models;
+	if (jumps)    { jumps->modLits += DL;}
+}
 ///////////////////////////////////////////////////////////////////////////////
 
 
 ///////////////////////////////////////////////////////////////////////////////
 // Clauses
 ///////////////////////////////////////////////////////////////////////////////
-//! Type for passing additional clause information.
+//! Type storing initial information on a (learnt) clause.
 class ClauseInfo {
 public:
-	enum { MAX_LBD = (1<<7)-1, MAX_ACTIVITY = (1<<15)-1 }; 
-	ClauseInfo(ConstraintType t = Constraint_t::static_constraint) 
-		: act_(0), lbd_(MAX_LBD), rootLbd_(MAX_LBD), type_(t), tag_(0) {}
+	typedef ClauseInfo self_type;
+	enum { MAX_LBD = Activity::MAX_LBD, MAX_ACTIVITY = (1<<22)-1 }; 
+	ClauseInfo(ConstraintType t = Constraint_t::static_constraint) : act_(0), lbd_(MAX_LBD), type_(t), tag_(0) {
+		static_assert(sizeof(self_type) == sizeof(uint32), "Unsupported padding");
+	}
+	bool           learnt()   const { return type() != Constraint_t::static_constraint; }
 	ConstraintType type()     const { return static_cast<ConstraintType>(type_); }
 	uint32         activity() const { return static_cast<uint32>(act_); }
 	uint32         lbd()      const { return static_cast<uint32>(lbd_); }
-	uint32         rootLbd()  const { return static_cast<uint32>(rootLbd_); }
 	bool           tagged()   const { return tag_ != 0; }
-	ClauseInfo& setType(ConstraintType t) { type_ = static_cast<uint32>(t); return *this; }
-	ClauseInfo& setActivity(uint32 act)   {
-		if (act > MAX_ACTIVITY) act = MAX_ACTIVITY;
-		act_ = act;
-		return *this;
-	}
-	ClauseInfo& setLbd(uint32 a_lbd, uint32 a_rootLbd) {
-		assert(a_lbd >= a_rootLbd);
-		if (a_lbd > static_cast<uint32>(MAX_LBD)) {
-			a_lbd    = static_cast<uint32>(MAX_LBD);
-			a_rootLbd= std::min(a_lbd, a_rootLbd);
-		}
-		lbd_     = a_lbd;
-		rootLbd_ = a_rootLbd;
-		return *this;
-	}
-	
-	ClauseInfo& tag() { tag_ = 1; return *this; }
+	self_type&     setType(ConstraintType t) { type_  = static_cast<uint32>(t); return *this; }
+	self_type&     setActivity(uint32 act)   { act_   = std::min(act, (uint32)MAX_ACTIVITY); return *this; }
+	self_type&     setTagged(bool b)         { tag_   = static_cast<uint32>(b); return *this; }
+	self_type&     setLbd(uint32 a_lbd)      { lbd_   = std::min(a_lbd, (uint32)MAX_LBD); return *this; }
 private:
-	uint32 act_     : 15; // Initial clause activity
-	uint32 lbd_     :  7; // Literal block distance in the range [0, MAX_LBD]
-	uint32 rootLbd_ :  7; // LBD w.r.t to current root level ([0, MAX_LBD])
-	uint32 type_    :  2; // Type of clause
-	uint32 tag_     :  1; // conditional clause?
+	uint32 act_ : 22; // Activity of clause
+	uint32 lbd_ :  7; // Literal block distance in the range [0, MAX_LBD]
+	uint32 type_:  2; // One of ConstraintType
+	uint32 tag_ :  1; // Conditional constraint?
 };
-
 //! (Abstract) base class for clause types.
 /*!
  * ClauseHead is used to enforce a common memory-layout for all clauses.
@@ -312,7 +283,7 @@ private:
  */
 class ClauseHead : public LearntConstraint {
 public:
-	enum { HEAD_LITS = 3, MAX_SHORT_LEN = 5, MAX_LBD = (1<<5)-1, TAGGED_CLAUSE = 1023};
+	enum { HEAD_LITS = 3, MAX_SHORT_LEN = 5, MAX_LBD = (1<<5)-1, TAGGED_CLAUSE = 1023, MAX_ACTIVITY = (1<<15)-1 };
 	explicit ClauseHead(const ClauseInfo& init);
 	// base interface
 	//! Propagates the head and calls updateWatch() if necessary.
@@ -322,7 +293,7 @@ public:
 	//! True if this clause currently is the antecedent of an assignment.
 	bool     locked(const Solver& s) const;
 	//! Returns the activity of this clause.
-	Activity activity() const       { return makeActivity(info_.data.act, info_.data.lbd); }
+	Activity activity() const       { return Activity(info_.data.act, info_.data.lbd); }
 	//! Halves the activity of this clause.
 	void     decreaseActivity()     { info_.data.act >>= 1; }
 	//! Downcast from LearntConstraint.
@@ -331,15 +302,16 @@ public:
 	// clause interface
 	typedef std::pair<bool, bool> BoolPair;
 	//! Increases activity.
-	void bumpActivity()     { info_.data.act += (info_.data.act != ClauseInfo::MAX_ACTIVITY); }
+	void bumpActivity()     { info_.data.act += (info_.data.act != MAX_ACTIVITY); }
 	//! Adds watches for first two literals in head to solver.
 	void attach(Solver& s);
 	//! Returns true if head is satisfied w.r.t current assignment in s.
 	bool satisfied(const Solver& s);
 	//! Conditional clause?
 	bool tagged() const     { return info_.data.key == uint32(TAGGED_CLAUSE); }
+	bool learnt() const     { return info_.data.type!= 0; }
 	uint32 lbd()  const     { return info_.data.lbd; }
-	void setLbd(uint32 lbd) { info_.data.lbd = std::min(lbd, (uint32)MAX_LBD); }
+	void lbd(uint32 x)      { info_.data.lbd = std::min(x, uint32(MAX_LBD)); }
 	//! Removes watches from s.
 	virtual void     detach(Solver& s);
 	//! Returns the size of this clause.
@@ -360,7 +332,9 @@ public:
 protected:
 	friend struct ClauseWatch;
 	bool         toImplication(Solver& s);
-	void         clearTagged(){ info_.data.key = 0; }
+	void         clearTagged()   { info_.data.key = 0; }
+	void         setLbd(uint32 x){ info_.data.lbd = x; }
+	bool         hasLbd() const  { return info_.data.type != Constraint_t::learnt_other || lbd() != MAX_LBD; }
 	//! Shall replace the watched literal at position pos with a non-false literal.
 	/*!
 	 * \pre pos in [0,1] 
@@ -375,14 +349,16 @@ protected:
 			uint32 idx;
 			void   init(uint32 size) {
 				if (size <= ClauseHead::MAX_SHORT_LEN){ sizeExt = idx = negLit(0).asUint(); }
-				else                                  { sizeExt = (size << 2) + 1; idx = 0; }
+				else                                  { sizeExt = (size << 3) + 1; idx = 0; }
 			}
-			bool   isSmall()   const    { return (sizeExt & 1u) == 0;  }
-			bool   contracted()const    { return (sizeExt & 3u) == 3u; }
-			uint32 size()      const    { return sizeExt >> 2; }
-			void   setSize(uint32 size) { sizeExt = (size << 2) | (sizeExt & 3u); }
-			void   markContracted()     { sizeExt |= 2u;  }
-			void   clearContracted()    { sizeExt &= ~2u; }
+			bool   isSmall()     const    { return (sizeExt & 1u) == 0u; }
+			bool   contracted()  const    { return (sizeExt & 3u) == 3u; }
+			bool   strengthened()const    { return (sizeExt & 5u) == 5u; }
+			uint32 size()        const    { return sizeExt >> 3; }
+			void   setSize(uint32 size)   { sizeExt = (size << 3) | (sizeExt & 7u); }
+			void   markContracted()       { sizeExt |= 2u;  }
+			void   markStrengthened()     { sizeExt |= 4u;  }
+			void   clearContracted()      { sizeExt &= ~2u; }
 		}               local;
 		uint32          lits[2];
 	}       data_;   // additional data
@@ -531,7 +507,7 @@ struct ReasonStore64 : PodVector<Antecedent>::type {
 class Assignment  {
 public:
 	typedef PodVector<uint32>::type     AssignVec;
-	typedef PodVector<uint8>::type      SavedVec;
+	typedef PodVector<uint8>::type      PhaseVec;
 	typedef bk_lib::detail::if_then_else<
 		sizeof(Constraint*)==sizeof(uint64)
 		, ReasonStore64
@@ -556,7 +532,9 @@ public:
 	//! Returns v's value in the three-valued assignment.
 	ValueRep          value(Var v) const { return ValueRep(assign_[v] & 3u); }
 	//! Returns v's previously saved value in the three-valued assignment.
-	ValueRep          saved(Var v) const { return v < saved_.size() ? saved_[v] : value_free; }
+	ValueRep          saved(Var v) const { return v < phase_.size() ? ValueRep(phase_[v] & 3u)  : value_free; }
+	//! Returns v's preferred value or value_free if not set.
+	ValueRep          pref(Var v)  const { return v < phase_.size() ? ValueRep(phase_[v] >> 2) : value_free; }
 	//! Returns the decision level on which v was assigned if value(v) != value_free.
 	uint32            level(Var v) const { return assign_[v] >> 4u; }
 	//! Returns the reason for v being assigned if value(v) != value_free.
@@ -623,7 +601,7 @@ public:
 	 */
 	void undoTrail(LitVec::size_type first, bool save) {
 		if (!save) { popUntil<&Assignment::clearValue>(trail[first]); }
-		else       { saved_.resize(numVars(), 0); popUntil<&Assignment::saveAndClear>(trail[first]); }
+		else       { phase_.resize(numVars(), 0); popUntil<&Assignment::saveAndClear>(trail[first]); }
 		front  = trail.size();
 	}
 	//! Undos the last assignment.
@@ -633,8 +611,12 @@ public:
 	Literal&last()       { return trail.back(); }
 	//! Sets val as "previous value" of v.
 	void setSavedValue(Var v, ValueRep val) {
-		if (saved_.size() <= v) saved_.resize(v+1, 0);
-		saved_[v] = val;
+		if (phase_.size() <= v) phase_.resize(numVars(), 0);
+		save(v, val);
+	}
+	void setPrefValue(Var v, ValueRep val) {
+		if (phase_.size() <= v) phase_.resize(numVars(), 0);
+		phase_[v] = (phase_[v] & 3u) | (val << 2);
 	}
 	/*!
 	 * \name Implementation functions
@@ -657,7 +639,8 @@ public:
 private:
 	Assignment(const Assignment&);
 	Assignment& operator=(const Assignment&);
-	void    saveAndClear(Var v) { saved_[v] = value(v); clearValue(v); }
+	void    save(Var v, ValueRep val) { phase_[v] = (phase_[v] & uint8(0xFC)) | val; }
+	void    saveAndClear(Var v)       { save(v, value(v)); clearValue(v); }
 	template <void (Assignment::*op)(Var v)>
 	void popUntil(Literal stop) {
 		Literal p;
@@ -668,7 +651,7 @@ private:
 	}
 	AssignVec assign_; // for each var: three-valued assignment
 	ReasonVec reason_; // for each var: reason for being assigned (+ optional data)
-	SavedVec  saved_;  // for each var: previous assignment
+	PhaseVec  phase_;  // for each var: previous assignment and fixed sign (if any)
 	uint32    eliminated_;
 };
 
