@@ -285,28 +285,24 @@ inline compose_2_2<OP1, OP2,OP3> compose22(const OP1& op1, const OP2& op2, const
 	return compose_2_2<OP1, OP2, OP3>(op1, op2, op3);
 }
 
-template <class T>
+
+template <class T, class D = DeleteObject>
 class SingleOwnerPtr {
 public:
 	explicit SingleOwnerPtr(T* ptr) : ptr_( set_bit(uintp(ptr),0) ) {}
-	~SingleOwnerPtr() {
-		if (is_owner()) { 
-			delete release();
-		}
-	}
+	~SingleOwnerPtr()                 { *this = 0; }
+	bool is_owner()   const { return test_bit(ptr_, 0); }
+	T*   get()        const { return (T*)clear_bit(ptr_, 0); }
+	T&   operator*()  const { return *get(); }
+	T*   operator->() const { return  get(); }
 	SingleOwnerPtr& operator=(T* ptr) {
-		SingleOwnerPtr t(ptr);
-		t.swap(*this);
+		if (ptr != get() && is_owner()) { D deleter; deleter(release()); }
+		ptr_ = set_bit(uintp(ptr),0);
 		return *this;
 	}
-	T& operator*()  const { return *get(); }
-	T* operator->() const { return  get(); }
-	T*    release()   { store_clear_bit(ptr_, 0); return get();  }
-	T*    get() const { return (T*)clear_bit(ptr_, 0); }
-	void swap(SingleOwnerPtr& o) {
-		std::swap(ptr_, o.ptr_);
-	}
-	bool is_owner() const { return test_bit(ptr_, 0); }
+	void swap(SingleOwnerPtr& o) { std::swap(ptr_, o.ptr_); }
+	T*   release()               { store_clear_bit(ptr_, 0); return get();  }
+	T*   acquire()               { store_set_bit(ptr_, 0);   return get();  }
 private:
 	SingleOwnerPtr(const SingleOwnerPtr&);
 	SingleOwnerPtr& operator=(const SingleOwnerPtr&);
