@@ -47,7 +47,7 @@ public:
 		return ret.local;
 	}
 	SharedLiterals* getShared() { 
-		std::lock_guard<tbb::spin_mutex> lock(mutex_);
+		Clasp::lock_guard<Clasp::spin_mutex> lock(mutex_);
 		return current_ ? current_->share() : 0;
 	}
 	bool shared() const { return shared_; }
@@ -55,15 +55,15 @@ private:
 	SharedLiterals* createShared(const LitVec& c) {
 		SharedLiterals* newShared = SharedLiterals::newShareable(c, Constraint_t::learnt_other, 2);
 		SharedLiterals* oldShared = current_;
-		{ std::lock_guard<tbb::spin_mutex> lock(mutex_);
+		{ Clasp::lock_guard<Clasp::spin_mutex> lock(mutex_);
 			current_ = newShared;
 		}
 		if (oldShared) oldShared->release();
 		return newShared;
 	}
-	tbb::spin_mutex mutex_;
-	SharedLiterals* current_;
-	bool            shared_;
+	Clasp::spin_mutex mutex_;
+	SharedLiterals*   current_;
+	bool              shared_;
 };
 /////////////////////////////////////////////////////////////////////////////////////////
 // CBConsequences::LocalConstraint
