@@ -939,17 +939,14 @@ bool SharedLitsClause::simplify(Solver& s, bool reinit) {
 }
 
 void SharedLitsClause::destroy(Solver* s, bool detachFirst) {
-	if (s && detachFirst) {
-		ClauseHead::detach(*s);
+	if (s) {
+		if (detachFirst) { ClauseHead::detach(*s); }
+		if (learnt())    { s->freeLearntBytes(32); }
 	}
 	data_.shared->release();
 	void* mem = this;
 	this->~SharedLitsClause();
-	if (s) {
-		// return memory to solver
-		s->freeLearntBytes(32);
-		s->freeSmall(mem);
-	}
+	if (s) { s->freeSmall(mem); }
 }
 
 uint32 SharedLitsClause::isOpen(const Solver& s, const TypeSet& x, LitVec& freeLits) {
@@ -1215,7 +1212,7 @@ bool LoopFormula::simplify(Solver& s, bool) {
 	// - remove all false bodies
 	// - find the watched bodies
 	for (i = 1; i != end_-1; ++i) {
-		assert( !s.isFalse(lits_[i]) || !lits_[i].watched() ); // otherwise should be asserting 
+		assert( !s.isFalse(lits_[i]) || !lits_[i].watched() || s.isTrue(lits_[other_]) );
 		if (!s.isFalse(lits_[i])) {
 			sat |= s.isTrue(lits_[i]);
 			if (i != j) { lits_[j] = lits_[i]; }
