@@ -1,5 +1,5 @@
 // 
-// Copyright (c) 2006-2007, Benjamin Kaufmann
+// Copyright (c) 2006-2012, Benjamin Kaufmann
 // 
 // This file is part of Clasp. See http://www.cs.uni-potsdam.de/clasp/ 
 // 
@@ -34,7 +34,7 @@ uint32 Constraint::estimateComplexity(const Solver&) const { return 1;  }
 ClauseHead* Constraint::clause()          { return 0; } 
 LearntConstraint::~LearntConstraint()     {}
 LearntConstraint::LearntConstraint()      {}
-LearntConstraint::Activity LearntConstraint::activity() const { return makeActivity(0,  (1u<<7)-1); }
+Activity LearntConstraint::activity() const{ return Activity(0,  (1u<<7)-1); }
 void LearntConstraint::decreaseActivity() {}
 ConstraintType LearntConstraint::type() const { return Constraint_t::learnt_conflict; }
 /////////////////////////////////////////////////////////////////////////////////////////
