@@ -44,25 +44,25 @@ public:
 		active_  = maxParties;
 	}
 	// Returns the current semaphore counter.
-	int  counter()   { std::lock_guard<std::mutex> lock(semMutex_); return counter_; }
+	int  counter()   { Clasp::lock_guard<Clasp::mutex> lock(semMutex_); return counter_; }
 	// Returns the number of parties required to trip this barrier.
-	int  parties()   { std::lock_guard<std::mutex> lock(semMutex_); return active_;  } 
+	int  parties()   { Clasp::lock_guard<Clasp::mutex> lock(semMutex_); return active_;  } 
 	// Returns true if all parties are waiting at the barrier
-	bool active()    { std::lock_guard<std::mutex> lock(semMutex_); return unsafe_active(); }
+	bool active()    { Clasp::lock_guard<Clasp::mutex> lock(semMutex_); return unsafe_active(); }
 	
 	// barrier interface
 	
 	// Increases the barrier count, i.e. the number of 
 	// parties required to trip this barrier.
 	void addParty() {
-		std::lock_guard<std::mutex> lock(semMutex_);
+		Clasp::lock_guard<Clasp::mutex> lock(semMutex_);
 		++active_;
 	}
 	// Decreases the barrier count and resets the barrier
 	// if reset is true. 
 	// PRE: the thread does not itself wait on the barrier
 	void removeParty(bool reset) {
-		std::unique_lock<std::mutex> lock(semMutex_);
+		Clasp::unique_lock<Clasp::mutex> lock(semMutex_);
 		assert(active_ > 0);
 		--active_;
 		if      (reset)           { unsafe_reset(0); }
@@ -74,13 +74,13 @@ public:
 	// Applications shall use this value to designate one thread as the
 	// leader that will eventually reset the barrier thereby unblocking the other threads.
 	bool wait() {
-		std::unique_lock<std::mutex> lock(semMutex_);
+		Clasp::unique_lock<Clasp::mutex> lock(semMutex_);
 		if (--counter_ >= 0) { counter_ = -1; }
 		return unsafe_wait(lock);
 	}
 	// Resets the barrier and unblocks any waiting threads.
 	void reset(uint32 semCount = 0) {
-		std::lock_guard<std::mutex> lock(semMutex_);
+		Clasp::lock_guard<Clasp::mutex> lock(semMutex_);
 		unsafe_reset(semCount);
 	}
 	// semaphore interface
@@ -91,7 +91,7 @@ public:
 	// Returns false to signal that all but the calling thread
 	// are currently blocked.
 	bool down() {
-		std::unique_lock<std::mutex> lock(semMutex_);
+		Clasp::unique_lock<Clasp::mutex> lock(semMutex_);
 		if (--counter_ >= 0) { return true; }
 		return !unsafe_wait(lock);
 	}
@@ -101,7 +101,7 @@ public:
 	void up() {
 		bool notify;
 		{
-			std::lock_guard<std::mutex> lock(semMutex_);
+			Clasp::lock_guard<Clasp::mutex> lock(semMutex_);
 			notify    = (++counter_ < 1);
 		}
 		if (notify) semCond_.notify_one();
@@ -109,7 +109,7 @@ public:
 private:
 	BarrierSemaphore(const BarrierSemaphore&);
 	BarrierSemaphore& operator=(const BarrierSemaphore&);
-	typedef std::condition_variable cv;
+	typedef Clasp::condition_variable cv;
 	bool    unsafe_active() const { return -counter_ >= active_; }
 	void    unsafe_reset(uint32 semCount) {
 		int prev = counter_;
@@ -117,7 +117,7 @@ private:
 		if (prev < 0) { semCond_.notify_all(); }
 	}
 	// Returns true for the leader, else false
-	bool    unsafe_wait(std::unique_lock<std::mutex>& lock) {
+	bool    unsafe_wait(Clasp::unique_lock<Clasp::mutex>& lock) {
 		assert(counter_ < 0);
 		// don't put the last thread to sleep!
 		if (!unsafe_active()) {
@@ -126,7 +126,7 @@ private:
 		return unsafe_active();
 	}	
 	cv         semCond_;  // waiting threads
-	std::mutex semMutex_; // mutex for updating counter
+	Clasp::mutex semMutex_; // mutex for updating counter
 	int        counter_;  // semaphore's counter
 	int        active_;   // number of active threads
 };
@@ -188,18 +188,18 @@ struct ParallelSolve::SharedData {
 	bool        allowRestart()       const { return !hasControl(forbid_restart_flag); }
 	bool        setControl(uint32 flags)   { return (fetch_and_or(control, flags) & flags) != flags; }
 	bool        clearControl(uint32 flags) { return (fetch_and_and(control, ~flags) & flags) == flags; }
-	ScheduleStrategy    globalR;     // global restart strategy
-	uint64              maxConflict; // current restart limit
-	SharedContext*      ctx;         // shared context object
-	const LitVec*       path;        // initial guiding path - typically empty
-	Timer<RealTime>     syncT;       // thread sync time
-	std::mutex          modelM;      // model-mutex 
-	BarrierSemaphore    workSem;     // work-semaphore
-	queue               workQ;       // work-queue
-	uint32              nextId;      // next solver id to use
-	std::atomic<int>    workReq;     // > 0: someone needs work
-	std::atomic<uint32> restartReq;  // == numThreads(): restart
-	std::atomic<uint32> control;     // set of active message flags
+	ScheduleStrategy      globalR;     // global restart strategy
+	uint64                maxConflict; // current restart limit
+	SharedContext*        ctx;         // shared context object
+	const LitVec*         path;        // initial guiding path - typically empty
+	Timer<RealTime>       syncT;       // thread sync time
+	Clasp::mutex            modelM;      // model-mutex 
+	BarrierSemaphore      workSem;     // work-semaphore
+	queue                 workQ;       // work-queue
+	uint32                nextId;      // next solver id to use
+	Clasp::atomic<int>    workReq;     // > 0: someone needs work
+	Clasp::atomic<uint32> restartReq;  // == numThreads(): restart
+	Clasp::atomic<uint32> control;     // set of active message flags
 };
 
 // post message to all threads
@@ -287,7 +287,7 @@ void ParallelSolve::addSolver(Solver& s, const SolveParams& p) {
 		s.stats.enableStats(shared_->ctx->master()->stats);
 		allocThread(id, s, p);
 		// start in new thread
-		std::thread x(std::mem_fun(&ParallelSolve::solveParallel), this, id);
+		Clasp::thread x(std::mem_fun(&ParallelSolve::solveParallel), this, id);
 		thread_[id]->setThread(x);
 	}
 }
@@ -571,7 +571,7 @@ bool ParallelSolve::backtrackFromModel(Solver& s) {
 		// grab lock - models must be processed sequentially
 		// in order to simplify printing and to avoid duplicates
 		// in all non-trivial enumeration modes
-		std::lock_guard<std::mutex> lock(shared_->modelM);
+		Clasp::lock_guard<Clasp::mutex> lock(shared_->modelM);
 		// first check if the model is still valid once all
 		// information is integrated into the solver
 		if (shared_->terminate() || !thread_[s.id()]->isModel(s)) {
