@@ -504,6 +504,10 @@ bool Preprocessor::reclassify(PrgAtomNode* a, uint32 atomId, uint32 diffLits) {
 				uint32 otherId = findEqBody(bn, hash.second);
 				if (otherId != varMax) {
 					mergeBodies(bodyId, otherId);
+					if (nodes_[otherId].sHead == 1 && !prg_->bodies_[otherId]->simplifyHeads(*prg_, *this, true)) {
+						return ok_ = false;
+					}
+					nodes_[otherId].sHead = 0;
 				}
 				else {
 					prg_->bodyIndex_.insert(ProgramBuilder::BodyIndex::value_type(hash.second, bodyId));
