@@ -1292,7 +1292,9 @@ void ProgramBuilder::transformIntegrity(uint32 maxAux) {
 	if (!integrity.empty() && (integrity.size() == 1 || (atoms_.size()/double(bodies_.size()) > 0.5 && integrity.size() / double(bodies_.size()) < 0.01))) {
 		bodyIndex_.clear();
 		frozen_ = false;
-		assert(atoms_[1]->value() == value_false);
+		uint32 F= 1;
+		while (atoms_[F]->value() != value_false) { ++F; }
+		assert(F < atoms_.size());
 		for (BodyList::size_type i = 0; i != integrity.size(); ++i) {
 			PrgBodyNode* b = integrity[i];
 			uint32 est = b->bound()*( b->sumWeights()-b->bound() );
@@ -1301,7 +1303,7 @@ void ProgramBuilder::transformIntegrity(uint32 maxAux) {
 			} 
 			maxAux -= est;
 			startRule(CONSTRAINTRULE, b->bound());
-			addHead(1);
+			addHead(F);
 			for (uint32 g = 0; g != b->size(); ++g) {
 				addToBody(b->goal(g).var(), !b->goal(g).sign());
 			}
