@@ -264,16 +264,13 @@ private:
 	weight_t*       sumW_;    // achievable weight for individual literals
 };
 
-PrgRuleTransform::PrgRuleTransform() : impl_(0) {}
-PrgRuleTransform::~PrgRuleTransform() { delete impl_; }
+PrgRuleTransform::PrgRuleTransform() {}
 
 uint32 PrgRuleTransform::transform(ProgramBuilder& prg, PrgRule& rule) {
 	if (rule.type() == CHOICERULE) {
 		return transformChoiceRule(prg, rule);
 	}
-	delete impl_;
-	impl_ = new Impl(prg, rule);
-	return impl_->transform();
+	return Impl(prg, rule).transform();
 }
 
 PrgRuleTransform::Impl::Impl(ProgramBuilder& prg, PrgRule& r)
