@@ -219,19 +219,21 @@ void SatElite::bceVeRemove(uint32 id, bool freeId, Var ev, bool blocked) {
 bool SatElite::preprocess(bool enumModels) {
 	struct Scope { SatElite* self; char type; ~Scope() { 
 		self->cleanUp();
-		self->reportProgress(type, 100,100);
+		self->reportProgress(Event(self, type, 100,100));
 	}} sc = {this, '*'};
 	Solver* s = ctx_->master();
 	// skip preprocessing if other constraints are UNSAT
 	if (!s->propagate()) return false;
 	// preprocess only if not too many vars are frozen
-	uint32 frozen = ctx_->stats().vars_frozen;
-	if (options.maxFrozen != 1.0 && frozen > 0) {
+	double frozen = 0.0;
+	double maxF   = std::min(1.0, options.maxFrozen/100.0);
+	if (maxF != 1.0 && (frozen = ctx_->stats().vars_frozen) != 0.0) {
 		for (LitVec::const_iterator it = s->trail().begin(), end = s->trail().end(); it != end; ++it) {
 			frozen -= (ctx_->frozen(it->var()));
 		}
+		frozen = std::min(1.0,  frozen / double(s->numFreeVars()));
 	}
-	if (!SatElite::limit((uint32)clauses_.size()) && (options.maxFrozen == 1.0 || frozen/double(s->numFreeVars()) <= options.maxFrozen)) {
+	if (!SatElite::limit((uint32)clauses_.size()) && frozen <= maxF) {
 		// 0. allocate & init state
 		occurs_   = new OccurList[ctx_->numVars()+1];
 		qFront_   = 0;
@@ -323,7 +325,7 @@ bool SatElite::backwardSubsume() {
 	while (qFront_ != queue_.size()) {
 		if ((qFront_ & 8191) == 0) {
 			if (timeout()) break;
-			if (queue_.size() > 1000) reportProgress('S', qFront_, queue_.size());
+			if (queue_.size() > 1000) reportProgress(Event(this, 'S', qFront_, queue_.size()));
 		}
 		if (peekSubQueue() == 0) { ++qFront_; continue; }
 		Clause& c = *popSubQueue();
@@ -615,7 +617,7 @@ bool SatElite::bce() {
 		occurs_[v].bce=0; 
 		if ((ops & 1023) == 0)   {
 			if (timeout())         { bce.clear(); return true; }
-			if ((ops & 8191) == 0) { reportProgress('B', ops, 1+bce.size()); }
+			if ((ops & 8191) == 0) { reportProgress(Event(this, 'B', ops, 1+bce.size())); }
 		}
 		if (!cutoff(v) && !bceVe(v, 0)) { return false; }
 	}
@@ -631,7 +633,7 @@ bool SatElite::eliminateVars() {
 		occ = occurs_[v].numOcc();
 		if ((ops & 1023) == 0)   {
 			if (timeout())         { elimHeap_.clear(); return true; }
-			if ((ops & 8191) == 0) { reportProgress('E', ops, 1+elimHeap_.size()); }
+			if ((ops & 8191) == 0) { reportProgress(Event(this, 'E', ops, 1+elimHeap_.size())); }
 		}
 		if (!cutoff(v) && !bceVe(v, occ)) {
 			return false;
@@ -698,7 +700,9 @@ void SatElite::extendModel(Assignment& assign, LitVec& unconstr) {
 	// compute values of eliminated vars / blocked literals by "unit propagating"
 	// eliminated/blocked clauses in reverse order
 	uint32 uv = 0;
-	if (!unconstr.empty()) {
+	uint32 us = unconstr.size();
+	uint32 uo = us;
+	if (us) {
 		// flip last unconstraint variable to get "next" model
 		unconstr.back() = ~unconstr.back();
 	}
@@ -712,11 +716,12 @@ void SatElite::extendModel(Assignment& assign, LitVec& unconstr) {
 			// implied values we first need to set them back to free
 			assign.clearValue(last);
 		}
-		if (uv != unconstr.size() && unconstr[uv].var() == last) {
+		if (uv != us && unconstr[uv].var() == last) {
 			// last is unconstraint w.r.t the current model -
 			// set remembered value
 			check    = false;
-			assign.setValue(unconstr[uv].var(), trueValue(unconstr[uv]));
+			assign.setValue(last, trueValue(unconstr[uv]));
+			assign.setReason(last, 0);
 			++uv;
 		}
 		do {
@@ -734,18 +739,30 @@ void SatElite::extendModel(Assignment& assign, LitVec& unconstr) {
 					assert(c.blocked() || assign.value(x.var()) != falseValue(x));
 					assign.clearValue(x.var());
 					assign.setValue(x.var(), trueValue(x));
+					assign.setReason(x.var(), Antecedent(x));
 					check = false;
 				}
 			}
 			r = r->next();
 		} while (r && (x = (*r)[0]).var() == last);
 		if (assign.value(last) == value_free) {
-			// last is unconstraint w.r.t the model. Assume last to true; remember it
+			// last seems unconstraint w.r.t the model. Assume last to true; remember it
 			// so that we can also enumerate the model containing ~last.
 			assign.setValue(last, value_true);
+			assign.setReason(last, 0);
 			unconstr.push_back(posLit(last));	
 		}
 	} while (r);
+	// check whether newly added unconstraint vars are really unconstraint w.r.t the model
+	// or if they are implied by some blocked clause.
+	LitVec::iterator j = unconstr.begin()+uo;
+	for (LitVec::iterator it = j, end = unconstr.end(); it != end; ++it) {
+		if (!it->sign() && assign.reason(it->var()).isNull()) {
+			*j++ = *it;
+		}
+	}
+	unconstr.erase(j, unconstr.end());
+	// pop all vars that were enumerated in both phases
 	while (!unconstr.empty() && unconstr.back().sign()) {
 		unconstr.pop_back();
 	}
