@@ -87,7 +87,7 @@ void ShortImplicationsGraph::ImplicationList::addLearnt(Literal p, Literal q) {
 				return;
 			}
 			else { 
-				std::this_thread::yield();
+				Clasp::this_thread::yield();
 			}
 		}
 		else {
