@@ -170,7 +170,8 @@ bool SolveAlgorithm::initPath(Solver& s, const LitVec& path, InitParams& params)
 }
 
 ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p, SolveLimits& lim) {
-	if (s.hasConflict()) return false;
+	if (s.hasConflict()) return value_false;
+	if (lim.reached())   return value_free;
 	struct  ConflictLimits {
 		uint64 restart; // current restart limit
 		uint64 reduce;  // current reduce limit
@@ -214,7 +215,7 @@ ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p, SolveLimits&
 	cLimit.global       = lim.conflicts;
 	cLimit.restart      = UINT64_MAX;
 	uint64& rsLimit     = p.restart.local() ? sLimit.local : cLimit.restart;
-	uint64 maxRestarts  = std::max(lim.restarts, uint64(1));
+	uint64 nRestart     = 0;
 	rsLimit             = rs.current();
 	if (p.restart.dynamic()) {
 		s.stats.enableQueue(rs.base);
@@ -254,8 +255,8 @@ ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p, SolveLimits&
 			progress.evType = SolvePathEvent::event_none;
 			if (rsLimit == 0 || sLimit.dynamicRestart(s.stats)) {
 				// restart reached - do restart
-				++s.stats.restarts;
-				if (p.restart.counterRestart && (s.stats.restarts % p.restart.counterRestart) == 0 ) {
+				++s.stats.restarts; ++nRestart;
+				if (p.restart.counterRestart && (nRestart % p.restart.counterRestart) == 0 ) {
 					inDegree.clear();
 					s.heuristic()->bump(s, inDegree, p.restart.counterBump / (double)s.inDegree(inDegree));
 				}
@@ -285,11 +286,11 @@ ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p, SolveLimits&
 					s.stats.queue->reset();
 				}
 				s.undoUntil(0);
-				if (rsLimit == 0)                { rsLimit = rs.next(); }
-				if (!minLimit)                   { minLimit= rs.current(); }
-				if (p.reduce.strategy.fRestart)  { db      = s.reduceLearnts(p.reduce.fRestart(), p.reduce.strategy); }
-				if (s.stats.restarts == shuffle) { shuffle+= p.restart.shuffleNext; s.shuffleOnNextSimplify();}
-				if (--maxRestarts == 0)          { break; }
+				if (rsLimit == 0)              { rsLimit = rs.next(); }
+				if (!minLimit)                 { minLimit= rs.current(); }
+				if (p.reduce.strategy.fRestart){ db      = s.reduceLearnts(p.reduce.fRestart(), p.reduce.strategy); }
+				if (nRestart == shuffle)       { shuffle+= p.restart.shuffleNext; s.shuffleOnNextSimplify();}
+				if (nRestart == lim.restarts)  { break; }
 				s.stats.cflLast = s.stats.analyzed;
 				progress.evType = SolvePathEvent::event_restart;
 			}	
@@ -315,7 +316,7 @@ ValueRep SolveAlgorithm::solvePath(Solver& s, const SolveParams& p, SolveLimits&
 	}
 	s.stats.cflLast = s.stats.analyzed - s.stats.cflLast;
 	if (lim.conflicts != UINT64_MAX) { lim.conflicts = cLimit.global; }
-	if (lim.restarts  != UINT64_MAX) { lim.restarts  = maxRestarts;   }
+	if (lim.restarts  != UINT64_MAX) { lim.restarts -= nRestart;   }
 	return result;
 }
 /////////////////////////////////////////////////////////////////////////////////////////
