@@ -467,7 +467,7 @@ void Solver::removeConditional() {
 		for (i = j = 0; i != end; ++i) {
 			ClauseHead* c = static_cast<LearntConstraint*>(learnts_[i])->clause();
 			if (!c || !c->tagged()) {
-				learnts_[j++] = c;
+				learnts_[j++] = learnts_[i];
 			}
 			else {
 				assert((decisionLevel() == rootLevel() || !c->locked(*this)) && "Solver::removeConditional(): must not remove locked constraint!");
@@ -485,7 +485,7 @@ void Solver::strengthenConditional() {
 		for (i = j = 0; i != end; ++i) {
 			ClauseHead* c = static_cast<LearntConstraint*>(learnts_[i])->clause();
 			if (!c || !c->tagged() || !c->strengthen(*this, p, true).second) {
-				learnts_[j++] = c;
+				learnts_[j++] = learnts_[i];
 			}
 			else {
 				assert((decisionLevel() == rootLevel() || !c->locked(*this)) && "Solver::strengthenConditional(): must not remove locked constraint!");
