@@ -190,41 +190,19 @@ Constraint::PropResult DefaultUnfoundedCheck::propagate(Solver& s, Literal, uint
 	assert(&s == solver_); (void)s;
 	uint32 index  = data >> 1;
 	uint32 type   = data & 1;
-	if (type == 0) {
-		// a body became false - remove sources if necessary
-		if (bodies_[index].watches > 0) {
-			invalid_.push_back(index);
-		}
-	}	
-	else {
-		// an atom literal became false
-		assert(index < watches_.size());
-		const ExtWatch& w = watches_[index];
-		if (w.bodyId == idMax) {
-			// atom is the head of a choice rule
-			// normally head false -> body false and hence the head has its source autmatically removed
-			// for choice rules we must force source removal explicity
-			uint32 bodyId = atoms_[w.data].watch();
-			if (atoms_[w.data].hasSource() && !s.isFalse(graph_->getBodyNode(bodyId).lit)) {
-				atoms_[w.data].markSourceInvalid();
-				sourceQ_.push_back(w.data);
-			}
-		}
-		else {
-			// a literal relevant to a body became false
-			const DependencyGraph::BodyNode& body = graph_->getBodyNode(w.bodyId);
-			assert(body.extended());
-			ExtData* ext = extended_[bodies_[w.bodyId].lower_or_ext];
-			if (test_bit(w.data, 0)) {
-				// p is external and false, remove from WS
-				ext->removeFromWs(w.data>>1, body.pred_weight(w.data>>1, true));
-			}
-			if (ext->lower > 0 && bodies_[w.bodyId].watches > 0 && !solver_->isFalse(graph_->getBodyNode(w.bodyId).lit)) {
-				// The body is not a valid source but at least one head atom 
-				// still depends on it:  mark body as invalid source
-				invalid_.push_back(w.bodyId);
-			}
-		}
+	if (type == 0) { // a body became false
+		// update source if necessary
+		if (bodies_[index].watches > 0) { invalid_.push_back(index); }
+	}
+	else if (watches_[index].bodyId == idMax) { // head of a choice rule became false
+		// normally head false -> body false and hence the head has its source 
+		// autmatically removed when the invalid body is processed;
+		// for choice rules we must force source removal explicity
+		const ExtWatch w = watches_[index];
+		invalidExt_.push_back((w.data << 1) | ExtWatch::watch_choice_false);
+	}
+	else { // a literal relevant to a body became false
+		invalidExt_.push_back((index << 1) | ExtWatch::watch_body_goal_false);
 	}
 	return PropResult(true, true);  // always keep the watch
 }
@@ -256,10 +234,7 @@ void DefaultUnfoundedCheck::reset() {
 		atoms_[unfounded_.vec_[i]].ufs = 0;
 	}
 	unfounded_.clear();
-	while (!sourceQ_.empty()) {
-		atoms_[sourceQ_.back()].resurrectSource();
-		sourceQ_.pop_back();
-	}
+	invalidExt_.clear();
 	invalid_.clear();
 	activeClause_.clear();
 }
@@ -302,20 +277,24 @@ void DefaultUnfoundedCheck::AddSource::operator()(const BodyNodeP& n, NodeId ato
 	assert(n.node->extended() && n.node->get_pred(idx) == atomId);
 	(void)atomId;
 	ExtData* ext = self->extended_[self->bodies_[n.id].lower_or_ext];
-	if (ext->lower > 0 && ext->addToWs(idx, n.node->pred_weight(idx, false))) {
+	if (ext->lower > 0 || self->bodies_[n.id].watches == 0) {
+		// currently not a source - safely add pred to our watch set
+		ext->addToWs(idx, n.node->pred_weight(idx, false));
+	}
+	if (!self->solver_->isFalse(n.node->lit) && ext->lower <= 0) {
+		// valid source - propagate to heads
 		self->forwardSource(n);
 	}
 }
 // an atom in extended body n has lost its source, check if n is no longer a valid source
 void DefaultUnfoundedCheck::RemoveSource::operator()(const BodyNodeP& n, NodeId atomId, uint32 idx) const {
 	assert(n.node->extended() && n.node->get_pred(idx) == atomId); (void)atomId;
-	ExtData* ext   = self->extended_[self->bodies_[n.id].lower_or_ext];
-	bool wasSource = ext->lower <= 0;
+	ExtData* ext = self->extended_[self->bodies_[n.id].lower_or_ext];
 	ext->removeFromWs(idx, n.node->pred_weight(idx, false));
-	if (wasSource && ext->lower > 0 && self->bodies_[n.id].watches > 0) {
+	if (ext->lower > 0 && self->bodies_[n.id].watches > 0) {
 		// extended bodies don't always become false if a predecessor becomes false
 		// eagerly enqueue all successors watching this body
-		self->forwardUnsource(n, addTodo || !self->solver_->isFalse(n.node->lit));
+		self->forwardUnsource(n, true);
 	}
 }
 
@@ -376,13 +355,38 @@ void DefaultUnfoundedCheck::removeSource(NodeId bodyId) {
 /////////////////////////////////////////////////////////////////////////////////////////
 bool DefaultUnfoundedCheck::findUnfoundedSet() {
 	// first: remove all sources that were recently falsified
-	if (!sourceQ_.empty()) {
-		propagateSource(true);
+	VarVec::size_type unpick = invalid_.size();
+	for (VarVec::size_type i = 0; i != invalidExt_.size(); ++i) {
+		uint32        id = invalidExt_[i] >> 1;
+		ExtWatch::Type t = static_cast<ExtWatch::Type>(invalidExt_[i] & 1u);
+		if (t == ExtWatch::watch_choice_false) {
+			if (atoms_[id].hasSource() && !solver_->isFalse(graph_->getBodyNode(atoms_[id].watch()).lit)) {
+				atoms_[id].markSourceInvalid();
+				sourceQ_.push_back(id);
+				propagateSource(true);
+			}
+		}
+		else if (t == ExtWatch::watch_body_goal_false) {
+			typedef DependencyGraph::BodyNode BodyNode;
+			const ExtWatch& w = watches_[id];
+			BodyData&       b = bodies_[w.bodyId];
+			ExtData*      ext = extended_[b.lower_or_ext];
+			const BodyNode& B = graph_->getBodyNode(w.bodyId);
+			NodeId       pred = w.data >> 1;
+			ext->removeFromWs(pred, B.pred_weight(pred, test_bit(w.data, 0) != 0));
+			if (ext->lower > 0 && b.watches && b.picked == 0 && !solver_->isFalse(B.lit)) {
+				invalid_.push_back(w.bodyId);
+				b.picked = 1;
+			}
+		}
 	}
-	for (VarVec::size_type i = 0; i != invalid_.size(); ++i) { 
-		removeSource(invalid_[i]); 
+	for (VarVec::size_type i = 0; i != invalid_.size(); ++i) {
+		removeSource(invalid_[i]);
 	}
-	invalid_.clear();
+	for (VarVec::size_type i = unpick; i != invalid_.size(); ++i) {
+		bodies_[invalid_[i]].picked = 0;
+	}
+	invalid_.clear(); invalidExt_.clear();
 	assert(sourceQ_.empty() && unfounded_.empty());
 	// second: try to re-establish sources.
 	while (!todo_.empty()) {
@@ -581,6 +585,7 @@ void DefaultUnfoundedCheck::createLoopFormula() {
 
 // computes the reason why a set of atoms is unfounded
 void DefaultUnfoundedCheck::computeReason() {
+	assert(reasonExt_.empty());
 	uint32 ufsScc = graph_->getAtomNode(unfounded_.front()).scc;
 	for (VarVec::size_type i = unfounded_.front_; i != unfounded_.vec_.size(); ++i) {
 		const DependencyGraph::AtomNode& atom = graph_->getAtomNode(unfounded_.vec_[i]);
@@ -591,8 +596,8 @@ void DefaultUnfoundedCheck::computeReason() {
 			}
 		}
 	}
-	for (VarVec::size_type i = 0; i != pickedExt_.size(); ++i) { bodies_[pickedExt_[i]].picked = 0; }
-	pickedExt_.clear();
+	for (VarVec::size_type i = 0; i != reasonExt_.size(); ++i) { bodies_[reasonExt_[i]].picked = 0; }
+	reasonExt_.clear();
 	info_     = ClauseInfo(Constraint_t::learnt_loop);
 	uint32 dl = solver_->finalizeConflictClause(activeClause_, info_);
 	assert(dl == solver_->decisionLevel() && "Loop nogood must contain a literal from current DL!");
@@ -625,7 +630,7 @@ void DefaultUnfoundedCheck::addIfReason(const BodyNodeP& n, uint32 uScc) {
 			}
 			else if (bodies_[n.id].picked == 0) {
 				bodies_[n.id].picked = 1;
-				pickedExt_.push_back(n.id);
+				reasonExt_.push_back(n.id);
 				// Check if the body depends on the atoms from the unfounded set. I.e.
 				// would the body still be false if all but its unfounded literals would be true?
 				ExtData* ext     = extended_[bodies_[n.id].lower_or_ext];
@@ -652,7 +657,7 @@ void DefaultUnfoundedCheck::addIfReason(const BodyNodeP& n, uint32 uScc) {
 	}
 	else if (n.node->scc == uScc && n.node->extended() && bodies_[n.id].picked == 0) {
 		bodies_[n.id].picked = 1;
-		pickedExt_.push_back(n.id);
+		reasonExt_.push_back(n.id);
 		// body is neither false nor a valid source - add all false lits to reason set
 		AddReasonLit addFalseLits = { this };
 		graph_->visitBodyLiterals(n, addFalseLits);
