@@ -30,8 +30,8 @@ struct SharedWeightLits {
 	SharedWeightLits(uint32 size, bool hasWeight) : shared_lits(size, true, hasWeight) {
 		shared_counter = 1; 
 	}
-	std::atomic<int> shared_counter;
-	WeightLits       shared_lits;
+	Clasp::atomic<int> shared_counter;
+	WeightLits         shared_lits;
 };
 
 SharedWeightLits* toSharedLits(WeightLits* x) {
