@@ -17,10 +17,7 @@
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-
-#include <cppunit/TestFixture.h>
-#include <cppunit/TestAssert.h>
-#include <cppunit/extensions/HelperMacros.h>
+#include "test.h"
 #include <algorithm>
 #include <clasp/clause.h>
 #include <clasp/solver.h>
@@ -413,7 +410,7 @@ public:
 		CPPUNIT_ASSERT_EQUAL(false, c->strengthen(*solver, posLit(7), false).second);
 		CPPUNIT_ASSERT_EQUAL(uint32(3), c->size());
 		CPPUNIT_ASSERT_EQUAL(true, c->strengthen(*solver, posLit(11)).second);
-		CPPUNIT_ASSERT_EQUAL(sizeof(Clause) + (9*sizeof(Literal)), ((Clause*)c)->computeAllocSize());
+		CPPUNIT_ASSERT_EQUAL(uint32(sizeof(Clause) + (9*sizeof(Literal))), ((Clause*)c)->computeAllocSize());
 	}
 
 	void testStrengthenBug() {
@@ -844,7 +841,7 @@ public:
 	}
 
 	void testLoopFormulaBodyConflict() {
-		LoopFormula* lf = lfTestInit();
+		lfTestInit();
 		solver->undoUntil(0);
 		
 		solver->assume( ~b3 );
@@ -864,7 +861,7 @@ public:
 		CPPUNIT_ASSERT(std::find(r.begin(), r.end(), ~b2) != r.end());
 	}
 	void testLoopFormulaAtomConflict() {
-		LoopFormula* lf = lfTestInit();
+		lfTestInit();
 		solver->undoUntil(0);
 		solver->assume( ~b3 );
 		solver->propagate();
@@ -892,7 +889,7 @@ public:
 	}
 
 	void testLoopFormulaDontChangeSat() {
-		LoopFormula* lf = lfTestInit();
+		lfTestInit();
 		solver->undoUntil(0);
 		CPPUNIT_ASSERT_EQUAL(true, solver->assume( ~b1 ) && solver->propagate());
 		CPPUNIT_ASSERT_EQUAL(true, solver->assume( ~b3 ) && solver->propagate());
@@ -953,7 +950,7 @@ public:
 	}
 
 	void testLoopFormulaPropTrueAtomInSatClause() {
-		LoopFormula* lf = lfTestInit();
+		lfTestInit();
 		solver->undoUntil(0);
 		solver->assume( ~a1 );
 		solver->propagate();
@@ -1000,7 +997,7 @@ private:
 	}
 	std::string toString(const LitVec& c) {
 		std::string res="[";
-		for (int i = 0; i != c.size(); ++i) {
+		for (uint32 i = 0; i != c.size(); ++i) {
 			if (c[i].sign()) {
 				res += '~';
 			}
