@@ -17,11 +17,7 @@
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-
-#include <cppunit/TestFixture.h>
-#include <cppunit/TestAssert.h>
-#include <cppunit/extensions/HelperMacros.h>
-#include "common.h"
+#include "test.h"
 #include <clasp/solver.h>
 #include <clasp/program_builder.h>
 #include <clasp/unfounded_check.h>
