@@ -17,11 +17,7 @@
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-
-#include <cppunit/TestFixture.h>
-#include <cppunit/TestAssert.h>
-#include <cppunit/extensions/HelperMacros.h>
-#include "common.h"
+#include "test.h"
 #include <clasp/solver.h>
 #include <clasp/program_builder.h>
 #include <clasp/minimize_constraint.h>
@@ -31,7 +27,7 @@ using namespace std;
 namespace Clasp { namespace Test {
 	struct ClauseObserver : public DecisionHeuristic {
 		Literal doSelect(Solver&){return Literal();}
-		void newConstraint(const Solver& s, const Literal* first, LitVec::size_type size, ConstraintType) {
+		void newConstraint(const Solver&, const Literal* first, LitVec::size_type size, ConstraintType) {
 			Clause c;
 			for (LitVec::size_type i = 0; i < size; ++i, ++first) {
 				c.push_back(*first);
@@ -82,7 +78,8 @@ class ProgramBuilderTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testSatBodyBug);
 	CPPUNIT_TEST(testAddUnknownAtomToMinimize);
 	CPPUNIT_TEST(testWriteWeakTrue);
-
+	CPPUNIT_TEST(testSimplifyBodyEqBug);
+	
 	CPPUNIT_TEST(testAssertEqSelfblocker);
 	CPPUNIT_TEST(testAddClauses);
 	CPPUNIT_TEST(testAddCardConstraint);
@@ -625,6 +622,25 @@ public:
 		CPPUNIT_ASSERT(str.str().find(bp) != std::string::npos);
 	}
 
+	void testSimplifyBodyEqBug() {
+		// {x,y,z}.
+		// a :- x,z.
+		// b :- x,z.
+		// c :- a, y, b.
+		builder.startProgram(ctx)
+			.setAtomName(1, "a").setAtomName(2, "b").setAtomName(3, "c").setAtomName(4, "x").setAtomName(5, "y").setAtomName(6, "z")
+			.startRule(CHOICERULE).addHead(4).addHead(5).addHead(6).endRule()
+			.startRule().addHead(1).addToBody(4, true).addToBody(6, true).endRule()
+			.startRule().addHead(2).addToBody(4, true).addToBody(6, true).endRule()
+			.startRule().addHead(3).addToBody(1, true).addToBody(5, true).addToBody(2, true).endRule()
+		; 
+		CPPUNIT_ASSERT_EQUAL(true, builder.endProgram() && ctx.endInit());
+		PrgAtomNode* a = builder.getAtom(1);
+		PrgAtomNode* b = builder.getAtom(2);
+		CPPUNIT_ASSERT(b->eq() && b->eqNode() == 1);
+		CPPUNIT_ASSERT(a->posDep.size() == 1);
+	}
+
 	void testAssertEqSelfblocker() {
 		builder.startProgram(ctx)
 			.setAtomName(1, "a").setAtomName(2, "x").setAtomName(3, "y").setAtomName(4, "q").setAtomName(5, "r")
