@@ -17,14 +17,10 @@
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-
-#include <cppunit/TestFixture.h>
-#include <cppunit/TestAssert.h>
-#include <cppunit/extensions/HelperMacros.h>
-#include <algorithm>
+#include "test.h"
 #include <clasp/solver.h>
 #include <clasp/clause.h>
-
+#include <algorithm>
 #ifdef _MSC_VER
 #pragma warning (disable : 4267) //  conversion from 'size_t' to unsigned int
 #pragma once
@@ -33,8 +29,8 @@
 
 namespace Clasp { namespace Test {
 using namespace Clasp::mt;
-class OtherClausesTest : public CppUnit::TestFixture {
-	CPPUNIT_TEST_SUITE(OtherClausesTest);
+class SharedClauseTest : public CppUnit::TestFixture {
+	CPPUNIT_TEST_SUITE(SharedClauseTest);
 	CPPUNIT_TEST(testClauseCtorAddsWatches);
 	CPPUNIT_TEST(testPropSharedClauseConflict);
 	CPPUNIT_TEST(testPropRandomClauses);
@@ -47,7 +43,7 @@ class OtherClausesTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testCloneShared);
 	CPPUNIT_TEST_SUITE_END(); 
 public:
-	OtherClausesTest() {
+	SharedClauseTest() {
 		a1 = posLit(ctx.addVar(Var_t::atom_var));
 		a2 = posLit(ctx.addVar(Var_t::atom_var));
 		a3 = posLit(ctx.addVar(Var_t::atom_var));
@@ -257,7 +253,7 @@ private:
 	}
 	std::string toString(const LitVec& c) {
 		std::string res="[";
-		for (int i = 0; i != c.size(); ++i) {
+		for (uint32 i = 0; i != c.size(); ++i) {
 			if (c[i].sign()) {
 				res += '~';
 			}
@@ -285,5 +281,5 @@ private:
 		makeLits( pos, size - pos ); 
 	}
 };
-CPPUNIT_TEST_SUITE_REGISTRATION(OtherClausesTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(SharedClauseTest);
 } } 
