@@ -17,14 +17,10 @@
 // along with Clasp; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
-
-#include <cppunit/TestFixture.h>
-#include <cppunit/TestAssert.h>
-#include <cppunit/extensions/HelperMacros.h>
+#include "test.h"
 #include <clasp/solver.h>
 #include <clasp/clause.h>
 #include <clasp/clasp_facade.h>
-#include "common.h"
 
 namespace Clasp { namespace Test {
 using namespace Clasp::mt;
@@ -39,13 +35,13 @@ struct TestingConstraint : public LearntConstraint {
 	void undoLevel(Solver&) {
 		++undos;
 	}
-	bool simplify(Solver& s, bool) { return sat; }
+	bool simplify(Solver&, bool) { return sat; }
 	void reason(Solver&, Literal, LitVec& out) { out = ante; }
 	void destroy(Solver* s, bool b) {
 		if (deleted) *deleted = true;
 		LearntConstraint::destroy(s, b);
 	}
-	bool locked(const Solver& s) const { return false; }
+	bool locked(const Solver&) const { return false; }
 	uint32 isOpen(const Solver&, const TypeSet&, LitVec&) { return 0; }
 	static uint32 size() { return uint32(10); }
 	ConstraintType type() const { return type_; }
@@ -77,6 +73,7 @@ struct TestingPostProp : public PostPropagator {
 class SolverTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST_SUITE(SolverTest);
 	CPPUNIT_TEST(testReasonStore);
+	CPPUNIT_TEST(testSingleOwnerPtr);
 	CPPUNIT_TEST(testDefaults);
 	CPPUNIT_TEST(testVarNullIsSentinel);
 	CPPUNIT_TEST(testSolverAlwaysContainsSentinelVar);
@@ -126,6 +123,7 @@ class SolverTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST(testResolveConflictBounded);
 
 	CPPUNIT_TEST(testClearAssumptions);
+	CPPUNIT_TEST(testStopConflict);
 
 	CPPUNIT_TEST(testSearchKeepsAssumptions);
 	CPPUNIT_TEST(testSearchAddsLearntFacts);
@@ -133,15 +131,18 @@ class SolverTest : public CppUnit::TestFixture {
 
 	CPPUNIT_TEST(testStats);
 	CPPUNIT_TEST(testIncrementalSolve);
-
+#if WITH_THREADS
 	CPPUNIT_TEST(testLearntShort);
 	CPPUNIT_TEST(testLearntShortAreDistributed);
+#endif
 
 	CPPUNIT_TEST(testUnfortunateSplitSeq);
 	CPPUNIT_TEST(testSplitInc);
 	CPPUNIT_TEST(testSplitFlipped);
 	CPPUNIT_TEST(testSplitFlipToNewRoot);
 	CPPUNIT_TEST(testSplitImplied);
+
+	CPPUNIT_TEST(testAddShortIncremental);
 	CPPUNIT_TEST_SUITE_END(); 
 public:
 	void setUp() {
@@ -182,9 +183,33 @@ public:
 		x->destroy();
 	}
 	void testReasonStore() {
-		testReasonStore<ReasonStore32>();
+		if (sizeof(void*) == sizeof(uint32)) {
+			testReasonStore<ReasonStore32>();
+		}
 		testReasonStore<ReasonStore64>();
 	}
+
+	void testSingleOwnerPtr() {
+		bool conDel1 = false, conDel2 = false;
+		TestingConstraint* f = new TestingConstraint(&conDel2);
+		{
+			SingleOwnerPtr<Constraint, DestroyObject> x(new TestingConstraint(&conDel1));
+			SingleOwnerPtr<Constraint, DestroyObject> y(f);
+			y.release();
+		}
+		CPPUNIT_ASSERT_EQUAL(true, conDel1);
+		CPPUNIT_ASSERT_EQUAL(false, conDel2);
+		{
+			SingleOwnerPtr<Constraint, DestroyObject> y(f);
+			y = f;
+			CPPUNIT_ASSERT_EQUAL(true, y.is_owner());
+			y.release();
+			CPPUNIT_ASSERT_EQUAL(false, conDel2);
+			y = f;
+			CPPUNIT_ASSERT_EQUAL(true, !conDel2 && y.is_owner());
+		}
+		CPPUNIT_ASSERT_EQUAL(true, conDel2);
+	}
 	void testDefaults() {
 		Solver& s = *ctx.master();
 		const SolverStrategies& x = s.strategies();
@@ -250,7 +275,7 @@ public:
 
 	void testEliminateVar() {
 		Var v1 = ctx.addVar(Var_t::atom_var);
-		Var v2 = ctx.addVar(Var_t::body_var);
+		ctx.addVar(Var_t::body_var);
 		Solver& s = ctx.startAddConstraints();
 		ctx.eliminate(v1);
 		CPPUNIT_ASSERT_EQUAL(uint32(2), s.numVars());
@@ -783,8 +808,7 @@ public:
 		Solver& s = ctx.startAddConstraints();
 		ClauseCreator cc(&s);
 		Literal tag = ctx.tagLiteral();
-		cc.start(Constraint_t::learnt_conflict).add(posLit(a)).add(posLit(b)).add(posLit(c)).add(~tag);
-		Constraint* newCon = cc.end().local;
+		cc.start(Constraint_t::learnt_conflict).add(posLit(a)).add(posLit(b)).add(posLit(c)).add(~tag).end();
 		CPPUNIT_ASSERT(s.numLearntConstraints() == 1);
 		s.strengthenConditional();
 		CPPUNIT_ASSERT(ctx.numLearntShort() == 1 || ctx.numTernary() == 1);
@@ -993,7 +1017,7 @@ public:
 		Var a = ctx.addVar( Var_t::atom_var );
 		Var b = ctx.addVar( Var_t::atom_var );
 		Var c = ctx.addVar( Var_t::atom_var );
-		Var d = ctx.addVar( Var_t::atom_var );
+		ctx.addVar( Var_t::atom_var );
 		Solver& s = ctx.startAddConstraints();
 		ctx.addBinary(posLit(a), negLit(b));
 		ctx.addBinary(negLit(a), posLit(b));
@@ -1010,7 +1034,7 @@ public:
 	void testClearAssumptions() {
 		Var a = ctx.addVar( Var_t::atom_var );
 		Var b = ctx.addVar( Var_t::atom_var );
-		Var c = ctx.addVar( Var_t::atom_var );
+		ctx.addVar( Var_t::atom_var );
 		Solver& s = ctx.startAddConstraints();
 		ctx.addBinary(negLit(a), negLit(b));
 		ctx.addBinary(negLit(a), posLit(b));
@@ -1026,6 +1050,32 @@ public:
 		CPPUNIT_ASSERT_EQUAL(false, s.clearAssumptions());		
 	}
 
+	void testStopConflict() {
+		Var a = ctx.addVar( Var_t::atom_var );
+		Var b = ctx.addVar( Var_t::atom_var );
+		Var c = ctx.addVar( Var_t::atom_var );
+		Var d = ctx.addVar( Var_t::atom_var );
+		Solver& s = ctx.startAddConstraints();
+		ctx.addBinary(negLit(a), negLit(b));
+		ctx.addBinary(negLit(a), posLit(b));
+		ctx.endInit();
+		s.assume(posLit(a)) && !s.propagate() && s.resolveConflict();
+		CPPUNIT_ASSERT(s.decisionLevel() == 0 && s.queueSize() == 1 && !s.hasConflict());
+		s.setStopConflict();
+		CPPUNIT_ASSERT(s.hasConflict() && !s.resolveConflict());
+		s.clearStopConflict();
+		CPPUNIT_ASSERT(s.decisionLevel() == 0 && s.queueSize() == 1 && !s.hasConflict());
+		s.propagate();
+		s.assume(posLit(c)) && s.propagate();
+		s.pushRootLevel(1);
+		CPPUNIT_ASSERT(s.rootLevel() == 1);
+		s.assume(posLit(d));
+		s.setStopConflict();
+		CPPUNIT_ASSERT(s.rootLevel() == 2);
+		s.clearStopConflict();
+		CPPUNIT_ASSERT(s.rootLevel() == 1 && s.queueSize() == 1);
+	}
+
 	void testStats() {
 		ProblemStats p1, p2;
 		CPPUNIT_ASSERT_EQUAL(uint32(0), p1.vars);
@@ -1214,14 +1264,14 @@ public:
 		Literal d = posLit(ctx.addVar( Var_t::atom_var ));
 		struct Dummy : public Distributor {
 			Dummy() : Distributor(), unary(0), binary(0), ternary(0) {}
-			void publish(const Solver& source, SharedLiterals* lits) {
+			void publish(const Solver&, SharedLiterals* lits) {
 				uint32 size = lits->size();
 				unary   += size == 1;
 				binary  += size == 2;
 				ternary += size == 3;
 				shared.push_back(lits);
 			}
-			uint32 receive(const Solver& in, SharedLiterals** out, uint32 num) { 
+			uint32 receive(const Solver&, SharedLiterals** out, uint32 num) { 
 				uint32 r = 0;
 				while (!shared.empty() && num--) {
 					out[r++] = shared.back();
@@ -1420,6 +1470,20 @@ public:
 		s.updateGuidingPath(gp, pos, impl);
 		CPPUNIT_ASSERT(std::find(gp.begin(), gp.end(), ~f) != gp.end());
 	}
+
+	void testAddShortIncremental() {
+		Literal a = posLit(ctx.addVar( Var_t::atom_var ));
+		Literal b = posLit(ctx.addVar( Var_t::atom_var ));
+		ctx.setSolvers(2);
+		ctx.startAddConstraints();
+		ctx.addBinary(a, b);
+		ctx.endInit();
+		CPPUNIT_ASSERT(ctx.numBinary()  == 1);
+		ctx.startAddConstraints();
+		ctx.addBinary(~a, ~b);
+		ctx.endInit();
+		CPPUNIT_ASSERT(ctx.numBinary()  == 2);
+	}
 private:
 	SharedContext ctx;
 	void integrateGp(Solver& s, LitVec& gp) {
@@ -1453,3 +1517,4 @@ private:
 };
 CPPUNIT_TEST_SUITE_REGISTRATION(SolverTest);
 } } 
+
