@@ -48,11 +48,14 @@ public:
 	Messages   messages; // n warnings and at most one error
 	int        help;     // print help and exit
 	bool       version;  // print version and exit
+	
+	typedef std::pair<DescriptionLevel, std::string> HelpOpt;
 private:
 	AppOptions(const AppOptions&);
 	AppOptions& operator=(const AppOptions&);
+	virtual HelpOpt initHelpOption() const { return HelpOpt(desc_level_default, "Print help information and exit"); }
 	virtual void initOptions(ProgramOptions::OptionContext& root) = 0;
-	virtual bool validateOptions(const ProgramOptions::ParsedOptions&, Messages&) = 0;
+	virtual bool validateOptions(const ProgramOptions::OptionContext& root, const ProgramOptions::ParsedOptions&, Messages&) = 0;
 	virtual void printHelp(const ProgramOptions::OptionContext& root) = 0;
 	virtual void printVersion(const ProgramOptions::OptionContext& root) = 0;
 };
