@@ -63,6 +63,7 @@ public:
 	const char*        argName()        const  { return value_->arg(); }
 	bool               assignDefault()  const;
 	std::size_t        maxColumn()      const;
+	DescriptionLevel   descLevel()      const  { return value_->level(); }
 private:
 	std::string name_;        // name (and unique key) of option
 	const char* description_; // description of the option (used for --help)
@@ -91,20 +92,17 @@ public:
 	/*!
 	 * Creates a new group of options under the given caption.
 	 */
-	OptionGroup(const std::string& caption = "");
-	
-	
+	OptionGroup(const std::string& caption = "", DescriptionLevel descLevel = desc_level_default);
 	~OptionGroup();
 
 	//! Returns the caption of this group.
 	const std::string& caption() const { return caption_; }
 
-	std::size_t size()    const { return options_.size();  }
-	bool        empty()   const { return options_.empty(); }
-	bool        hidden()  const { return hide_; }
-	void        hidden(bool b)  { hide_ = b; }
-	option_iterator begin() const { return options_.begin(); }
-	option_iterator end()   const { return options_.end();   }
+	std::size_t      size()     const { return options_.size();  }
+	bool             empty()    const { return options_.empty(); }
+	option_iterator  begin()    const { return options_.begin(); }
+	option_iterator  end()      const { return options_.end();   }
+	DescriptionLevel descLevel()const { return level_; }
 	
 	//! Returns an object that can be used to add options.
 	/*!
@@ -124,22 +122,34 @@ public:
 	//! Adds option to this group.
 	void addOption(std::auto_ptr<Option> option);
 
-	//! Creates a formated description of all options in this group.
-	void format(OptionOutput& out, size_t maxW) const;
+	void setDescriptionLevel(DescriptionLevel level) { level_ = level; }
 
-	std::size_t maxColumn() const;
+	//! Creates a formated description of all options with level() <= level in this group.
+	void format(OptionOutput& out, size_t maxW, DescriptionLevel level = desc_level_default) const;
+
+	std::size_t maxColumn(DescriptionLevel level) const;
 private:
 	friend class OptionContext;
-	std::string caption_;
-	OptionList  options_;
-	bool        hide_;
+	std::string      caption_;
+	OptionList       options_;
+	DescriptionLevel level_;
 };
 
 class OptionInitHelper {
 public:
 	explicit OptionInitHelper(OptionGroup& owner);
-	OptionInitHelper& operator()(const char* name,
-		Value* val, const char* description);
+	
+	//! Factory function for creating an option.
+	/*!
+	 * \param key <name>[!][,<alias>][,@<level>]
+	 * \param val  Value of the option
+	 * \param desc Description of the option
+	 * 
+	 * \note If <name> is followed by an exclamation mark ('!')
+	 *       the option is marked as negatable.
+	 */
+	OptionInitHelper& operator()(const char* key,
+		Value* val, const char* desc);
 private:
 	OptionGroup* owner_;
 };
@@ -165,7 +175,7 @@ public:
 	//! Type for identifying an option within a context
 	typedef OptionList::const_iterator      option_iterator;
 	
-	OptionContext(const std::string& caption = "");
+	OptionContext(const std::string& caption = "", DescriptionLevel desc_default = desc_level_default);
 	~OptionContext();
 
 	const std::string& caption() const { return caption_; }
@@ -193,11 +203,11 @@ public:
 	
 	option_iterator begin() const { return options_.begin(); }
 	option_iterator end()   const { return options_.end();   }
-
+	
 	//! Returns the number of options in this context.
-	std::size_t size() const   { return options_.size(); }
+	std::size_t    size()   const { return options_.size(); }
 	//! Returns the number of groups in this context
-	std::size_t groups() const { return groups_.size(); }
+	std::size_t    groups() const { return groups_.size(); }
 
 	enum FindType { find_name = 1, find_prefix = 2, find_name_or_prefix = find_name|find_prefix, find_alias = 4 };
 	
@@ -221,17 +231,25 @@ public:
 	 * UnknownOption or AmbiguousOption.
 	 */
 	option_iterator tryFind(const char* key, FindType t = find_name) const;
+
+	const OptionGroup& findGroup(const std::string& caption) const;
+	const OptionGroup* tryFindGroup(const std::string& caption) const;
 	
-	//! Writes a formatted description of all options in this context.
+	//! Sets the description level to be used when generating description.
 	/*!
-	 * \note Hidden groups are ignored unless addHidden is true.
+	 * Once set, functions generating descriptions will only consider groups
+	 * and options with description level <= std::min(level, desc_level_all).
 	 */
-	OptionOutput& description(OptionOutput& out, bool addHidden = false) const;
+	void             setActiveDescLevel(DescriptionLevel level);
+	DescriptionLevel getActiveDescLevel() const { return descLevel_; }
+	
+	//! Writes a formatted description of options in this context.
+	OptionOutput& description(OptionOutput& out) const;
 	
 	//! Returns the default command-line of this context.
 	std::string defaults(std::size_t prefixSize = 0) const;
 	
-	//! Writes a formatted description of all options in this context to os.
+	//! Writes a formatted description of options in this context to os.
 	friend std::ostream& operator<<(std::ostream& os,  const OptionContext& ctx);
 
 	//! Assigns any default values to all options not in exclude.
@@ -242,12 +260,13 @@ public:
 private:
 	PrefixRange findImpl(const char* key, FindType t) const;
 	void        insertOption(size_t groupId, const SharedOptPtr& o);
-	size_t      findGroup(const std::string& name) const;
+	size_t      findGroupKey(const std::string& name) const;
 	
-	Name2Key    index_;
-	OptionList  options_;
-	GroupList   groups_;
-	std::string caption_;
+	Name2Key         index_;
+	OptionList       options_;
+	GroupList        groups_;
+	std::string      caption_;
+	DescriptionLevel descLevel_;
 };
 
 class OptionParser;
@@ -315,17 +334,17 @@ private:
 //! Base class for options parsers.
 class OptionParser {
 public:
-	enum OptionType {short_opt, long_opt, end_opt, no_opt};
-	OptionParser(OptionContext& o, bool allowUnreg);
+	typedef OptionContext::FindType FindType;
+	OptionParser(const OptionContext& o, bool allowUnreg);
 	virtual ~OptionParser();
 	ParsedValues parse();
 protected:
-	OptionType getOptionType(const char* o) const;
-	SharedOptPtr getOption(const char* name, OptionType t) const;
 	// PRE: key.get() != 0
 	void addOptionValue(const SharedOptPtr& key, const std::string& value) {
 		pv_.add(key, value);
 	}
+	SharedOptPtr getOption(const char* name, FindType ft) const;
+	SharedOptPtr tryGetOption(const char* name, FindType ft) const;
 private:
 	virtual void doParse() = 0;
 	ParsedValues pv_;
@@ -431,7 +450,7 @@ typedef bool (*PosOption)(const std::string&, std::string&);
 * \throw UnknownOption if allowUnregistered is false and an argument is found
 * that does not match any option.
 */
-ParsedValues parseCommandLine(int& argc, char** argv, OptionContext& ctx,
+ParsedValues parseCommandLine(int& argc, char** argv, const OptionContext& ctx,
 	bool allowUnregistered = true,
 	PosOption posParser = 0);
 
@@ -447,7 +466,7 @@ ParsedValues parseCommandLine(int& argc, char** argv, OptionContext& ctx,
 * \throw SyntaxError if command line syntax is incorrect.
 * \throw UnknownOption if an argument is found that does not match any option.
 */
-ParsedValues parseCommandString(const std::string& cmd, OptionContext& ctx, bool allowUnreg = false, PosOption posParser = 0);
+ParsedValues parseCommandString(const std::string& cmd, const OptionContext& ctx, bool allowUnreg = false, PosOption posParser = 0);
 
 /*!
 * Parses a config file having the format key = value.
@@ -462,7 +481,7 @@ ParsedValues parseCommandString(const std::string& cmd, OptionContext& ctx, bool
 * \note Keys are option's long names.
 * \note Lines starting with # are treated as comments and are ignored.
 */
-ParsedValues parseCfgFile(std::istream& is, OptionContext& o, bool allowUnregistered);
+ParsedValues parseCfgFile(std::istream& is, const OptionContext& o, bool allowUnregistered);
 
 }
 
