@@ -34,6 +34,16 @@ namespace std { using ::size_t; }
 
 namespace ProgramOptions {
 
+
+enum DescriptionLevel {
+	desc_level_default = 0, /**< Always shown in description */
+	desc_level_e1      = 1,
+	desc_level_e2      = 2,
+	desc_level_e3      = 3,
+	desc_level_all     = 4,
+	desc_level_hidden  = 5 /**< Never shown in description */
+};
+
 //! Manages the value of an option and defines how it is parsed from a string.
 /*!
  * The library maintains a 1:1-relationship between options and their values.
@@ -42,13 +52,13 @@ namespace ProgramOptions {
  */
 class Value {
 public:
-	//! Possible states of an option value
+	//! Possible (tentative) states of an option value.
 	enum State { 
 		value_unassigned = 0, /**< no value assigned */
 		value_defaulted  = 1, /**< a default value is assigned */
-		value_parsed     = 2  /**< a parsed value is assigned */
+		value_fixed      = 2  /**< a parsed value is assigned */
 	};
-	//! Possible value descriptions
+	//! Possible value descriptions.
 	enum DescType {
 		  desc_name    = 1
 		, desc_default = 2
@@ -56,7 +66,7 @@ public:
 	};
 	virtual ~Value();
 
-	//! Returns the current state of this value
+	//! Returns the current state of this value.
 	State state() const { return static_cast<State>(state_); }
 	
 	/*!
@@ -67,7 +77,7 @@ public:
 		return this;
 	}
 
-	//! Returns the name of this value
+	//! Returns the name of this value.
 	/*!
 	 * \note The default name is "<arg>" unless isFlag() is true in which
 	 *       case the default is "".
@@ -75,9 +85,29 @@ public:
 	const char* arg()          const;
 	Value*      arg(const char* n) { return desc(desc_name, n); }
 
-	//! Sets an alias name for the option of this value
+	//! Sets an alias name for the corresponding option.
 	Value* alias(char c) { optAlias_ = c; return this; }
 	char   alias() const { return optAlias_; }
+	//! Sets a description level for the corresponding option.
+	/*!
+	 * Description levels can be used to suppress
+	 * certain options when generating option descriptions.
+	 */
+	Value* level(DescriptionLevel lev) {
+		unsigned x = (lev * level_shift);
+		flags_     = x | (flags_&(level_shift-1));
+		return this;
+	}
+	//! Returns the description level of the corresponding option.
+	DescriptionLevel level() const { return DescriptionLevel(flags_ / level_shift); }
+
+	//! Returns true if this is the value of an negatable option.
+	/*!
+	 * If an option '--option' is negatable, passing '--no-option' 
+	 * on the command-line will set the value of '--option' to 'no'.
+	 */
+	bool isNegatable() const { return hasProperty(property_negatable); }
+	Value* negatable()       { setProperty(property_negatable); return this; }
 
 	
 	//! Returns true if value can be implicitly created from an empty string.
@@ -95,9 +125,7 @@ public:
 	//! Returns true if this is the value of an option flag.
 	/*!
 	 * Similar to isImplicit but with the difference that
-	 * an explicit value is only used if given via
-	 * the option name (i.e. '--option=value' will use 'value' 
-	 * but '-ovalue' will use the implicit value).
+	 * no value is accepted on the command-line.
 	 *
 	 * Used for options like '--help' or '--version'.
 	 */
@@ -109,6 +137,7 @@ public:
 	 */
 	Value* flag() { setProperty(property_flag); return this; }
 
+	
 	//! Returns true if the value of this option can be composed from multiple source.
 	bool isComposing() const { return hasProperty(property_composing); }
 	/*!
@@ -130,14 +159,14 @@ public:
 	 * \see bool Value::isImplicit() const
 	 */
 	Value* implicit(const char* str) { return desc(desc_implicit, str); }
-	//! Returns the implicit value of this or 0 if isImplicit() == false
+	//! Returns the implicit value of this or 0 if isImplicit() == false.
 	const char* implicit() const;
 
-	//! Parses the given string and updates the value's state
+	//! Parses the given string and updates the value's state.
 	/*!
-	 * \param name  The name of the option associated with this value
-	 * \param value The value to parse
-	 * \param st    The state to which the value should transition if parsing is succesful
+	 * \param name  The name of the option associated with this value.
+	 * \param value The value to parse.
+	 * \param st    The state to which the value should transition if parsing is succesful.
 	 *   
 	 * \return 
 	 * - true if the given string contains a valid value
@@ -145,14 +174,16 @@ public:
 	 *
 	 * \post if true is returned, state() is st
 	 */
-	bool parse(const std::string& name, const std::string& value, State st = value_parsed);
+	bool parse(const std::string& name, const std::string& value, State st = value_fixed);
 protected:
 	typedef unsigned char byte_t;
 	enum Property {
 		  property_implicit    = 1 // implicit value?
 		, property_flag        = 3 // implicit and type bool?
 	  , property_composing   = 4 // multiple values allowed?
-		, property_location    = 8 // fixed storage location?
+		, property_negatable   = 8 // negatable form allowed? 
+		, property_location    =16 // fixed storage location?
+		, not_a_property       =32
 	};
 	Value(byte_t flagSet, State initial = value_unassigned);
 	void setProperty(Property f)     { flags_ |= byte_t(f); }
@@ -163,7 +194,7 @@ protected:
 	const char* desc(DescType t) const;
 	Value*      desc(DescType t,  const char* d);
 private:
-	enum { desc_pack    = 8 };
+	enum { desc_pack    = 8, level_shift = not_a_property, levels = 255/level_shift };
 	byte_t state_;       // state: one of State
 	byte_t flags_;       // flag set holding properties
 	byte_t descFlag_;    // either desc_pack or one of DescType
