@@ -23,6 +23,10 @@
 #include <cctype>
 #include <limits.h>
 #include <cstring>
+#ifdef _MSC_VER
+#pragma warning (disable : 4996)
+#endif
+#include <stdio.h>
 
 using namespace ProgramOptions;
 using namespace std;
@@ -33,13 +37,15 @@ namespace ProgramOptions {
 bool AppOptions::parse(int argc, char** argv, const char* appName, ProgramOptions::PosOption p) {
 	ParsedOptions parsed;
 	try {
-		OptionContext  allOpts(std::string(appName).append(" Options"));
+		OptionContext allOpts(std::string(appName).append(" Options"));
+		HelpOpt       helpO = initHelpOption();
 		help    = 0;
 		version = false;
 		messages.clear();
+		Value* hv = helpO.first == desc_level_default ? storeTo(help)->flag() : storeTo(help)->arg("<n>")->implicit("1");
 		OptionGroup basic("Basic Options");
 		basic.addOptions()
-			("help,h"   , storeTo(help)->implicit("1"),  "Print help information and exit (2=extended help)")
+			("help,h"   , hv,  helpO.second.c_str())
 			("version,v", flag(version),   "Print version information and exit")    
 		;
 		allOpts.add(basic);
@@ -47,14 +53,22 @@ bool AppOptions::parse(int argc, char** argv, const char* appName, ProgramOption
 		parsed.assign(parseCommandLine(argc, argv, allOpts, false, p));
 		allOpts.assignDefaults(parsed);
 		if (help) {
+			if (help < 0 || (help-1) > (int)helpO.first) { 
+				char buf[128];
+				sprintf(buf, "%d", help);
+				throw ValueError(allOpts.caption(), ValueError::invalid_value, "help", buf);
+			}
+			DescriptionLevel x = (DescriptionLevel)std::min(help-1, (int)desc_level_all);
+			allOpts.setActiveDescLevel(x);
 			printHelp(allOpts);
 			return true;
 		}
+		help = 0;
 		if (version) {
 			printVersion(allOpts);
 			return true;
 		}
-		return validateOptions(parsed, messages);
+		return validateOptions(allOpts, parsed, messages);
 	}
 	catch(const std::exception& e) {
 		messages.error = e.what();
