@@ -25,7 +25,8 @@
 #include <program_opts/value_parser.h>
 #include <cctype>
 #include <algorithm>
-#include <stdlib.h>
+#include <iterator>
+#include <cstdlib>
 #include <limits>
 #include <cassert>
 #include <cstring>
